from pydantic import BaseModel


class BaseResource(BaseModel):
    class Config:
        populate_by_name = True
        frozen = False
        allow = "allow"  # TODO should this be extra = 'allow'

    @property
    def info(self):
        return self.dict()

    @property
    def _raw(self, exclude_fields: dict = None):
        exclude_fields = exclude_fields if exclude_fields else {}
        assert isinstance(exclude_fields, dict)
        return self.dict(by_alias=True, exclude=exclude_fields)

    # helper function for backwards compatibility ?
    @classmethod
    def create_from_dict(cls, d):
        if "line" in d:
            return cls.parse_obj(d["line"])
        else:
            return cls.parse_obj(d)



from .betting import (
    ActiveSport,
    ActiveRegion,
    ActiveCompetition,
    ActiveFixture,
    ActiveMarketType,
    ActiveMarket,
    ActiveSelection,
    ActiveBookmaker,
    Balance,
    Price,
    BetRequestCreate,
    BetRequest,
    BetRequestMatch,
    BetHistory,
    BetHistoryRequest,
    SelectionsForMarket,
    ActiveBet,
    ActiveBetRequests,
    Viewed,
    BetRequestMatchMore,
    PricesBookmaker,
    MyActiveBet,
    MyBetsBetRequests,
    MyBetsBets,
    BetRequestStop,
    CustomerOrderRef,
    CustomerStrategyRef,
    LockBet,
    LineMarketsSelectionsForMarket,
)
from .filters import (
    Filter,
    GetBetRequestFilter,
    CreateBetRequestBySelectionFilter,
    CreateBetRequestByCompetitorFilter,
    CreateBetRequestFilter,
)

from .account import Login, Token, AccountPreferences

from .messages import BaseRequestException, ResponseMessage



from __future__ import annotations
import hashlib
from typing import List, Union, TYPE_CHECKING
from uuid import UUID

if TYPE_CHECKING:
    from betconnect.resources import SelectionsForMarket

# "https://stackoverflow.com/questions/19989481/how-to-determine-if-a-string-is-a-valid-v4-uuid"
def is_valid_uuid(uuid_to_test, version=4):
    """
    Check if uuid_to_test is a valid UUID.

     Parameters
    ----------
    uuid_to_test : str
    version : {1, 2, 3, 4}

     Returns
    -------
    `True` if uuid_to_test is a valid UUID, otherwise `False`.

     Examples
    --------
    >>> is_valid_uuid('c9bf9e57-1685-4c89-bafb-ff5af830be8a')
    True
    >>> is_valid_uuid('c9bf9e58')
    False
    """

    try:
        uuid_obj = UUID(uuid_to_test, version=version)
    except ValueError:
        return False
    return str(uuid_obj) == uuid_to_test


def calculate_book_percentage(market_selections: List[SelectionsForMarket]) -> float:
    """
    Calculates the book % for a given market and runner
    :param List[SelectionsForMarket] market_selections: List of available selections for the market
    :return: A float value. Anything close to 1.0 is good. Anything 1.1-1.15 is a large markup
    """

    book_value = 0
    for selection in market_selections:
        if selection.trading_status == "Trading":
            if selection.max_price:
                if selection.max_price > 1.01:
                    book_value += 1 / selection.max_price
    return round(book_value, 3)


def parse_bet_request_id(bet_request_id: Union[str, UUID]) -> UUID:
    """
    Parses a string or UUID to a UUID
    :param bet_request_id: a string or UUID
    :return: a UUID valid for a bet_request_id
    """
    if isinstance(bet_request_id, UUID):
        return bet_request_id
    elif isinstance(bet_request_id, str):
        return UUID(bet_request_id)


def create_cheap_hash(txt: str, length: int = 15) -> str:
    # This is just a hash for debugging purposes.
    #    It does not need to be unique, just fast and short.
    # https://stackoverflow.com/questions/14023350
    hash_ = hashlib.sha1()
    hash_.update(txt.encode())
    return hash_.hexdigest()[:length]



import json
import pickle
import os

# noinspection SpellCheckingInspection
basepath = os.path.dirname(__file__)


def build_path(path):
    return basepath + "/" + path


def save_json_to_file(file_location, data):
    with open(file_location, "w") as f:
        json.dump(data, f)


def save_data_to_pickle_file(file_location, data):
    with open(file_location, "wb") as f:
        pickle.dump(data, f)


def load_json(file_name):
    with open(file_name, "r") as f:
        data = f.read()
        return json.loads(data)


def load_pickle(file_name):
    with open(file_name, "rb") as f:
        data = f.read()
        return pickle.loads(data)



from decouple import config
from betconnect.apiclient import APIClient
from betconnect.enums import Environment

# Create a trading client instance
client = APIClient(
    username=config("STAGING_BETCONNECT_USERNAME"),
    password=config("STAGING_BETCONNECT_PASSWORD"),
    api_key=config("STAGING_BETCONNECT_API_KEY"),
    environment=Environment.STAGING,
    personalised_production_url=config("PRODUCTION_URI"),
)

# login
client.account.login()

# get active sports
active_sports = client.betting.active_sports()

try:
    football = [s for s in active_sports if s.name == "Football"][0]
except IndexError:
    raise Exception("Cannot football as an active sport.")

# find active regions
active_regions = client.betting.active_regions(sport_id=football.sport_id)

# get the england region
try:
    active_region = [r for r in active_regions if r.name == "England"][0]
except IndexError:
    raise Exception("Cannot find the England region for football!")

# get active competitions
active_competitions = client.betting.active_competitions(
    sport_id=football.sport_id, region_id=active_region.region_id
)

# get market types
market_types = client.betting.active_market_types(sport_id=football.sport_id)

# get the Match result type
try:
    match_result_market_type = [t for t in market_types if t.name == "Match Result"][0]
except IndexError:
    raise Exception("Cannot find Match Result as a match type for football!")

for competition in active_competitions:
    # get active fixtures
    active_fixtures = client.betting.active_fixtures(
        sport_id=football.sport_id,
        competition_id=competition.competition_id,
        region_id=active_region.region_id,
    )

    for active_fixture in active_fixtures:

        # active markets
        active_markets = client.betting.active_markets(
            fixture_id=active_fixture.fixture_id
        )

        for active_market in active_markets:

            # get selections and prices for active market
            selections_for_market = client.betting.selections_for_market(
                fixture_id=active_fixture.fixture_id,
                market_type_id=active_market.market_type_id,
            )

# get a football line market
try:
    over_under_goals = [
        t for t in market_types if t.name == "Total Goals Over / Under"
    ][0]
except IndexError:
    raise Exception(
        "Cannot find Total Goals Over / Under as a match type for football!"
    )

for competition in active_competitions:
    # get active fixtures
    active_fixtures = client.betting.active_fixtures(
        sport_id=football.sport_id,
        competition_id=competition.competition_id,
        region_id=active_region.region_id,
    )

    for active_fixture in active_fixtures:
        # get active markets
        active_markets = client.betting.active_markets(
            fixture_id=active_fixture.fixture_id
        )

        for active_market in [
            m
            for m in active_markets
            if m.market_type_id == over_under_goals.market_type_id
        ]:
            # get selections for the line market
            selections_for_market = client.betting.selections_for_market(
                fixture_id=active_fixture.fixture_id,
                market_type_id=active_market.market_type_id,
            )

            # get a handicap market and active selections
            try:
                handicap_market = [
                    m for m in active_markets if m.is_handicap == "True"
                ][0]
                handicap_active_selections = client.betting.active_selections(
                    fixture_id=active_fixture.fixture_id,
                    market_type_id=active_market.market_type_id,
                    handicap=handicap_market.handicap,
                )
                # get individual prices
                for selection in handicap_active_selections:
                    selection_price = client.betting.prices(
                        fixture_id=active_fixture.fixture_id,
                        market_type_id=active_market.market_type_id,
                        competitor=selection.competitor,
                        handicap=handicap_market.handicap,
                    )

            except IndexError as e:
                raise Exception("Cannot find a handicap market.")



from betconnect import resources
from datetime import datetime, time
import pytest
from betconnect import exceptions
from uuid import UUID


class TestBettingResources:
    def test_active_bookmaker(self):
        active_bookmaker = resources.ActiveBookmaker(
            name="10Bet", bookmaker_id=6122, order=99, active=1
        )
        assert active_bookmaker.name == "10Bet"
        assert active_bookmaker.bookmaker_id == 6122
        assert active_bookmaker.order == 99
        assert active_bookmaker.active == 1

    def test_active_sport(self):
        active_sport = resources.ActiveSport(
            id=21,
            sport_id=14,
            name="Horse Racing",
            display_name="Horse Racing",
            slug="horse-racing",
            order=1,
            active=1,
            rate=2.0,
        )
        assert active_sport.id == 21
        assert active_sport.sport_id == 14
        assert active_sport.name == "Horse Racing"
        assert active_sport.display_name == "Horse Racing"
        assert active_sport.slug == "horse-racing"
        assert active_sport.order == 1
        assert active_sport.active == 1
        assert active_sport.rate == 2.0
        assert active_sport.bets_available is None

    def test_active_region(self):
        active_region = resources.ActiveRegion(
            name="England", region_id=3795032, iso="england", order=2
        )
        assert active_region.name == "England"
        assert active_region.region_id == 3795032
        assert active_region.iso == "england"
        assert active_region.order == 2

    def test_active_competition(self):
        active_competition = resources.ActiveCompetition(
            name="Southwell",
            display_name="Southwell",
            competition_id=840,
            active=1,
            order=9999,
        )
        assert active_competition.name == "Southwell"
        assert active_competition.display_name == "Southwell"
        assert active_competition.competition_id == 840
        assert active_competition.active == 1
        assert active_competition.order == 9999

    def test_active_market_type(self):
        active_market_type = resources.ActiveMarketType(
            market_type_id=6, name="WIN", active=1
        )
        assert active_market_type.market_type_id == 6
        assert active_market_type.name == "WIN"
        assert active_market_type.active == 1

    def test_active_market(self):
        active_market = resources.ActiveMarket(
            name="Race Winner",
            display_name="Race Winner",
            trading_status="Open",
            is_handicap="False",
            source_market_id="102347558",
            market_type_id=6,
            order=1,
            bet_types=["Win", "Each Way (1/4 odds 1-2-3)"],
        )
        assert active_market.name == "Race Winner"
        assert active_market.display_name == "Race Winner"
        assert active_market.trading_status == "Open"
        assert active_market.is_handicap == "False"
        assert active_market.source_market_id == "102347558"
        assert active_market.market_type_id == 6
        assert active_market.order == 1
        assert active_market.handicap is None
        assert active_market.bet_types == ["Win", "Each Way (1/4 odds 1-2-3)"]

    def test_balance(self):
        balance = resources.Balance(balance=1000)
        assert balance.balance == 1000
        assert balance.balance_uk_pounds == 10

    def test_prices_bookmaker(self):
        prices_bookmaker = resources.PricesBookmaker(id="123", name="Betfair")
        assert prices_bookmaker.id == "123"
        assert prices_bookmaker.name == "Betfair"

    def test_price(self):
        price = resources.Price(
            price="10",
            numerator="10",
            denominator="1",
            bookmakers=[{"id": "123", "name": "Betfair"}],
        )

        assert price.price == "10"
        assert price.numerator == 10
        assert price.denominator == 1
        assert price.bookmakers[0].id == "123"
        assert price.bookmakers[0].name == "Betfair"

    def test_active_selection(self):
        active_selection = resources.ActiveSelection(
            name="Metal Man",
            trading_status="NonRunner",
            selection_id="308685953",
            ut="2021-12-17 15:49:20.068318",
            competitor="1095697",
        )
        assert active_selection.name == "Metal Man"
        assert active_selection.trading_status == "NonRunner"
        assert active_selection.selection_id == "308685953"
        assert isinstance(active_selection.ut, datetime)
        assert active_selection.competitor == "1095697"

    def test_active_fixture(self):
        active_fixture = resources.ActiveFixture(
            fixture_id="8757184",
            display_name="Kempton Park",
            start_date="2021-12-17",
            time="16:15",
            each_way_active="1",
        )
        assert active_fixture.fixture_id == 8757184
        assert active_fixture.display_name == "Kempton Park"
        assert isinstance(active_fixture.start_date, datetime)
        assert active_fixture.time == time(16, 15)
        assert active_fixture.start_date_time == datetime(2021, 12, 17, 16, 15)
        assert active_fixture.each_way_active == "1"

    def test_bet_request(self):
        bet_request = resources.BetRequest(
            sport_name="Horse Racing",
            sport_id="14",
            competition_name="Kempton Park",
            region_name="England",
            start_time_utc="2021-12-17 19:45:00+00:00",
            fixture_name="Kempton Park",
            market_name="Race Winner",
            selection_name="Starter For Ten",
            price={
                "decimal": "3.25",
                "fraction": {"numerator": "9", "denominator": "4"},
            },
            fixture_id=8757185,
            market_type_id=6,
            competitor="1207634",
            bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
            bet_type="Win",
            requested_stake=5.0,
            liability=11.25,
            locked_stake=0.0,
            backer_stats={
                "strike_rate": 0.33,
                "roi": -100.0,
                "bet_requests": "10-50",
                "recent_performance": "L,L,L,L,L,L,L,L",
            },
            others_viewing_bet=0,
            lockable=True,
        )
        assert bet_request.sport_name == "Horse Racing"
        assert bet_request.sport_id == "14"
        assert bet_request.competition_name == "Kempton Park"
        assert bet_request.region_name == "England"
        assert isinstance(bet_request.start_time_utc, datetime)
        assert bet_request.fixture_name == "Kempton Park"
        assert bet_request.market_name == "Race Winner"
        assert bet_request.selection_name == "Starter For Ten"
        assert isinstance(bet_request.price, resources.Price)
        assert bet_request.fixture_id == 8757185
        assert bet_request.market_type_id == 6
        assert bet_request.competitor == "1207634"
        assert bet_request.bet_request_id == UUID(
            "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369"
        )
        assert bet_request.bet_type == "Win"
        assert bet_request.requested_stake == 5.0
        assert bet_request.liability == 11.25
        assert bet_request.locked_stake == 0.0
        assert bet_request.others_viewing_bet == 0
        assert bet_request.lockable

    def test_bet_request_create(self):
        bet_request_create = resources.BetRequestCreate(
            bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
            created="2021-12-17 16:20:49.113386",
            debit_stake=5,
            debit_commission=0.1,
        )
        assert bet_request_create.bet_request_id == UUID(
            "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369"
        )
        assert isinstance(bet_request_create.created, datetime)
        assert bet_request_create.debit_stake == 5
        assert bet_request_create.debit_commission == 0.1

    def test_active_bet(self):
        active_bet = resources.ActiveBet(
            bet_request_id="45e1a024-523a-4b25-a33d-4466775d0aac",
            bet_type_name="Win",
            competition_name="Hamilton Park",
            created_at="2021-07-15 09:38:22",
            fill_percentage=0.0,
            fixture_id=8248855,
            fixture_name="Hamilton Park",
            fixture_start_date="2021-07-15 12:00:00",
            market_name="Race Winner",
            matched_stake=0.0,
            price=1.16,
            price_denominator=25,
            price_numerator=4,
            selection_name="Misty Ayr",
            sport_name="Horse Racing",
            stake=10.0,
            status_name="Active",
        )
        assert active_bet.bet_request_id == UUID("45e1a024-523a-4b25-a33d-4466775d0aac")
        assert active_bet.bet_type_name == "Win"
        assert active_bet.competition_name == "Hamilton Park"
        assert isinstance(active_bet.created_at, datetime)
        assert active_bet.fill_percentage == 0.0
        assert active_bet.fixture_id == 8248855
        assert active_bet.fixture_name == "Hamilton Park"
        assert isinstance(active_bet.fixture_start_date, datetime)
        assert active_bet.market_name == "Race Winner"
        assert active_bet.matched_stake == 0.0
        assert active_bet.price == 1.16
        assert active_bet.price_denominator == 25
        assert active_bet.price_numerator == 4
        assert active_bet.selection_name == "Misty Ayr"
        assert active_bet.sport_name == "Horse Racing"
        assert active_bet.stake == 10.0
        assert active_bet.status_name == "Active"

    def test_bet_request_match(self):
        bet_request_match = resources.BetRequestMatch(
            matched=True,
            available=True,
            bet_request_id="45e1a024-523a-4b25-a33d-4466775d0aac",
            bet_id="124",
            bet_status="Active",
            amount_matched=10.0,
        )
        assert bet_request_match.matched
        assert bet_request_match.available
        assert bet_request_match.bet_request_id == UUID(
            "45e1a024-523a-4b25-a33d-4466775d0aac"
        )
        assert bet_request_match.bet_id == "124"
        assert bet_request_match.bet_status == "Active"
        assert bet_request_match.amount_matched == 10.0

    def test_bet_history(self):
        bet_history = resources.BetHistory(
            bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
            bet_request_status="Active",
            bet_type_name="Win",
            competition_name="Kempton Park",
            fill_percentage=0.0,
            fixture_name="Kempton Park",
            fixture_start_date="2021-12-17 19:45:00",
            market_name="Race Winner",
            matched_stake=0.0,
            price=3.25,
            price_denominator=4,
            price_numerator=9,
            region_iso="england",
            region_name="England",
            selection_name="Starter For Ten",
            sport_name="Horse Racing",
            stake=5.0,
        )

        assert bet_history.bet_request_id == UUID(
            "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369"
        )
        assert bet_history.bet_request_status == "Active"
        assert bet_history.bet_type_name == "Win"
        assert bet_history.competition_name == "Kempton Park"
        assert bet_history.fill_percentage == 0.0
        assert bet_history.fixture_name == "Kempton Park"
        assert isinstance(bet_history.fixture_start_date, datetime)
        assert bet_history.market_name == "Race Winner"
        assert bet_history.matched_stake == 0.0
        assert bet_history.price == 3.25
        assert bet_history.price_denominator == 4
        assert bet_history.price_numerator == 9
        assert bet_history.region_iso == "england"
        assert bet_history.region_name == "England"
        assert bet_history.selection_name == "Starter For Ten"
        assert bet_history.sport_name == "Horse Racing"
        assert bet_history.stake == 5.0

    def test_active_bets_request(self):
        active_bets_requests = resources.ActiveBetRequests(
            bets=[], bets_active=10, last_page=10, total_bets=20
        )

        assert active_bets_requests.bets == []
        assert active_bets_requests.bets_active == 10
        assert active_bets_requests.last_page == 10
        assert active_bets_requests.total_bets == 20

    def test_bet_history_request(self):
        bet_history_request = resources.BetHistoryRequest(
            bets=[], last_page=10, total_bets=20
        )
        assert bet_history_request.bets == []
        assert bet_history_request.last_page == 10
        assert bet_history_request.total_bets == 20

    def test_selections_for_market(self):
        selections_for_market = resources.SelectionsForMarket(
            source_fixture_id="8757185",
            source_market_id="102347567",
            source_market_type_id="6",
            source_selection_id="308686039",
            trading_status="Trading",
            name="Starter For Ten",
            competitor_id="1207634",
            ut="2021-12-16 08:53:29.827063",
            order=0,
            max_price=3.25,
            prices=[
                {
                    "price": "3.25",
                    "numerator": "9",
                    "denominator": "4",
                    "bookmakers": [
                        {"id": "4", "name": "William Hill"},
                        {"id": "6002", "name": "Skybet"},
                    ],
                },
                {
                    "price": "3.20",
                    "numerator": "11",
                    "denominator": "5",
                    "bookmakers": [
                        {"id": "3", "name": "Paddy Power"},
                        {"id": "6316", "name": "Betfair Sportsbook"},
                    ],
                },
                {
                    "price": "3.00",
                    "numerator": "2",
                    "denominator": "1",
                    "bookmakers": [
                        {"id": "6006", "name": "Coral"},
                        {"id": "6001", "name": "BetVictor"},
                        {"id": "17", "name": "Ladbrokes"},
                    ],
                },
            ],
        )
        assert selections_for_market.source_fixture_id == "8757185"
        assert selections_for_market.source_market_id == "102347567"
        assert selections_for_market.source_market_type_id == "6"
        assert selections_for_market.trading_status == "Trading"
        assert selections_for_market.name == "Starter For Ten"
        assert selections_for_market.competitor_id == "1207634"
        assert isinstance(selections_for_market.ut, datetime)
        assert selections_for_market.order == 0
        assert selections_for_market.max_price == 3.25
        for p in selections_for_market.prices:
            assert isinstance(p, resources.Price)

    def test_viewed(self):
        viewed = resources.Viewed(
            prev="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
            next="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5589",
        )

        assert viewed.prev == UUID("0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369")
        assert viewed.next == UUID("0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5589")

    def test_bet_request_match_more(self):
        bet_request_match_more = resources.BetRequestMatchMore(
            matched=True,
            available=True,
            viewed={
                "prev": "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
                "next": "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5589",
            },
        )
        assert bet_request_match_more.matched
        assert bet_request_match_more.available
        assert isinstance(bet_request_match_more, resources.BetRequestMatchMore)

    def test_bet_request_stop(self):
        bet_request_stop = resources.BetRequestStop(pending=False)
        assert bet_request_stop.pending is False

    def test_my_active_bet(self):
        my_active_bet = resources.MyActiveBet(
            actioned_at="2021-07-15 09:35:26",
            bet_created=1,
            bet_request_id="a575ba8b-2264-4f14-8f59-9eeb805e5a37",
            bet_request_status_id=1,
            bet_request_user_id="a6a1eb91-6217-4e0c-b759-fbcaa2b7a8ac",
            bet_type_name="Win",
            competition_name="Hamilton Park",
            count_in_place=None,
            fill_percentage=0.0,
            fixture_name="Hamilton Park",
            fixture_start_date="2021-07-15 12:00:00",
            market_name="Race Winner",
            matched_stake=0,
            price={"decimal": "1.16", "fraction": {"denominator": 25, "numerator": 4}},
            profit=0,
            region_iso="scotland",
            region_name="Scotland",
            selection_name="Misty Ayr",
            sport_external_id=14,
            sport_name="Horse Racing",
            sport_slug="horse-racing",
            stake=1000,
            status_name="Active",
            status_slug="active",
        )
        assert isinstance(my_active_bet.actioned_at, datetime)
        assert my_active_bet.adjustment is None
        assert my_active_bet.adjustment_type is None
        assert my_active_bet.bet_created == 1
        assert my_active_bet.bet_request_id == UUID(
            "a575ba8b-2264-4f14-8f59-9eeb805e5a37"
        )
        assert my_active_bet.bet_request_status_id == 1
        assert my_active_bet.bet_request_user_id == UUID(
            "a6a1eb91-6217-4e0c-b759-fbcaa2b7a8ac"
        )
        assert my_active_bet.bet_type_name == "Win"
        assert my_active_bet.competition_name == "Hamilton Park"
        assert my_active_bet.count_in_place is None
        assert my_active_bet.customer_order_ref is None
        assert my_active_bet.customer_strategy_ref is None
        assert my_active_bet.each_way_factor is None
        assert my_active_bet.fill_percentage == 0.0
        assert my_active_bet.fixture_name == "Hamilton Park"
        assert isinstance(my_active_bet.fixture_start_date, datetime)
        assert my_active_bet.handicap is None
        assert my_active_bet.market_name == "Race Winner"
        assert my_active_bet.matched_stake == 0
        assert isinstance(my_active_bet.price, resources.Price)
        assert my_active_bet.profit == 0
        assert my_active_bet.profit_loss is None
        assert my_active_bet.region_iso == "scotland"
        assert my_active_bet.region_name == "Scotland"
        assert my_active_bet.selection_name == "Misty Ayr"
        assert my_active_bet.sport_external_id == 14
        assert my_active_bet.sport_name == "Horse Racing"
        assert my_active_bet.sport_slug == "horse-racing"
        assert my_active_bet.stake == 1000
        assert my_active_bet.status_name == "Active"
        assert my_active_bet.status_slug == "active"
        assert my_active_bet.sub_account_id is None

        # Each Way Bet
        my_active_bet = resources.MyActiveBet(
            actioned_at="2022-01-16 11:43:16",
            bet_created=1,
            bet_request_id="9983e005-8301-4ef4-bbe9-ce441a3f491b",
            bet_request_status_id=1,
            bet_request_user_id="91358194-70a7-4d51-bb40-3f3ffeb386d6",
            bet_type_name="Each Way (1/5 odds 1-2-3)",
            competition_name="Southwell",
            count_in_place=None,
            customer_order_ref=None,
            customer_strategy_ref=None,
            each_way_factor=5,
            fill_percentage=0,
            fixture_name="Southwell",
            fixture_startdate="2022-01-16 16:00:00",
            handicap=None,
            market_name="Race Winner",
            matched_stake=0,
            price={"decimal": "6.0", "fraction": {"denominator": 1, "numerator": 5}},
            profit=0,
            region_iso="england",
            region_name="England",
            selection_name="Asadjumeirah",
            sport_external_id=14,
            sport_name="Horse Racing",
            sport_slug="horse-racing",
            stake=500,
            status_name="Active",
            status_slug="active",
            sub_account_id=None,
        )
        assert my_active_bet.bet_type_name == "Each Way (1/5 odds 1-2-3)"
        assert my_active_bet.each_way_factor == 5

        # Partially matched handicap bet
        my_active_bet = resources.MyActiveBet(
            actioned_at="2022-01-17 15:39:30",
            bet_created=1,
            bet_request_id="9613dbd6-fe44-437f-a64e-04b2f05701e9",
            bet_request_status_id=1,
            bet_request_user_id="91358194-70a7-4d51-bb40-3f3ffeb386d6",
            bet_type_name=None,
            competition_name="NFL",
            count_in_place=None,
            customer_order_ref=None,
            customer_strategy_ref=None,
            each_way_factor=None,
            fill_percentage=40,
            fixture_name="Los Angeles Rams v Arizona Cardinals",
            fixture_startdate="2022-01-18 01:15:00",
            handicap="-3.5",
            market_name="Point Spread",
            matched_stake=200,
            price={"decimal": "1.333", "fraction": {"denominator": 3, "numerator": 1}},
            profit=66,
            region_iso="us",
            region_name="United States of America",
            selection_name="Asadjumeirah",
            sport_external_id=17,
            sport_name="American Football",
            sport_slug="american-football",
            stake=500,
            status_name="Active",
            status_slug="active",
            sub_account_id=None,
        )
        assert my_active_bet.bet_type_name is None
        assert my_active_bet.each_way_factor is None
        assert my_active_bet.fill_percentage == 40
        assert my_active_bet.handicap == "-3.5"
        assert my_active_bet.matched_stake == 200
        assert my_active_bet.profit == 66

    def test_customer_order_ref(self):
        customer_order_ref_gen = resources.CustomerOrderRef.generate_random_order_ref()
        customer_order_ref = resources.CustomerOrderRef(
            customer_order_ref=customer_order_ref_gen
        )
        assert customer_order_ref.customer_order_ref == customer_order_ref_gen
        assert str(customer_order_ref) == customer_order_ref_gen

        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            resources.CustomerOrderRef(
                customer_order_ref="111111111111111111111111111111111111111111111111111"
            )

        # test customer order ref generate random UUID
        assert isinstance(
            UUID(resources.CustomerOrderRef.generate_random_order_ref()), UUID
        )

        customer_order_ref = resources.CustomerOrderRef.create_customer_order_ref(
            customer_order_ref="12344"
        )
        assert customer_order_ref.customer_order_ref == "12344"

        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            resources.CustomerOrderRef.create_customer_order_ref(customer_order_ref="")

        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            resources.CustomerOrderRef.create_customer_order_ref(
                customer_order_ref="11111111111111111111111111111111111111111"
            )

    def test_customer_strategy_ref(self):
        customer_strategy_ref_value = "best_strategy"
        customer_strategy_ref = resources.CustomerStrategyRef(
            customer_strategy_ref=customer_strategy_ref_value
        )
        assert (
            customer_strategy_ref.customer_strategy_ref == customer_strategy_ref_value
        )
        assert str(customer_strategy_ref) == customer_strategy_ref_value

        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            resources.CustomerStrategyRef(
                customer_strategy_ref="111111111111111111111111111111111111111111111111111"
            )

        customer_stratey_ref = (
            resources.CustomerStrategyRef.create_customer_strategy_ref("1234")
        )
        assert customer_stratey_ref.customer_strategy_ref == "1234"
        customer_stratey_ref = (
            resources.CustomerStrategyRef.create_customer_strategy_ref("123 4")
        )
        assert customer_stratey_ref.customer_strategy_ref == "1234"

        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            resources.CustomerStrategyRef.create_customer_strategy_ref(
                "12343333333333333333333333"
            )

        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            resources.CustomerStrategyRef.create_customer_strategy_ref("")

    def test_line_markets_selections_for_market(self):
        line_markets = resources.LineMarketsSelectionsForMarket(
            name="Over/Under (1.5)",
            display_name="Over/Under (1.5)",
            handicap="1.5",
            line_data=[
                {
                    "source_fixture_id": "8172709",
                    "source_market_id": "102340516",
                    "source_market_type_id": "259",
                    "source_selection_id": "308656048",
                    "trading_status": "Trading",
                    "name": "Over 1.5",
                    "outcome": "Over",
                    "ut": "2021-12-19 04:56:46.590564",
                    "handicap": 1.5,
                    "order": 1,
                    "prices": [
                        {
                            "price": "6.00",
                            "numerator": "5",
                            "denominator": "1",
                            "bookmakers": [
                                {"id": "6006", "name": "Coral"},
                                {"id": "17", "name": "Ladbrokes"},
                            ],
                        }
                    ],
                }
            ],
        )
        assert line_markets.name == "Over/Under (1.5)"
        assert line_markets.display_name == "Over/Under (1.5)"
        assert line_markets.handicap == "1.5"
        assert isinstance(line_markets.line_data, list)
        for market in line_markets.line_data:
            assert isinstance(market, resources.SelectionsForMarket)



from requests.sessions import Session
from datetime import datetime, timedelta
from .enums import Environment
from typing import Union, Optional
from betconnect import resources
from betconnect import config
from betconnect import exceptions
import logging

logger = logging.getLogger(__name__)


class BaseClient:
    def __init__(
        self,
        username: str,
        password: str,
        api_key: str,
        personalised_production_url: str,
        environment: Environment = Environment.PRODUCTION,
        session: Session = None,
    ):
        """
        :param username: your betconnect username (string)
        :param password: your betconnect password (string)
        :param api_key: your betconnect api key (string)
        :param personalised_production_url: A production user supplied url. (account manager will supply this)
        :param environment: the environment endpoint you want to send requests to (enum Environment)
        :param session: Session object used in request default None.Session created if None with auth and headers handled.
        """
        self._username = username
        self._password = password
        self._api_key = api_key
        self._environment = environment
        self._update_client_session(session)
        self._personalised_production_url = personalised_production_url
        self._set_endpoint_uris(environment)
        self.login_date: Optional[datetime] = None
        self.login_expiry_check: Optional[datetime] = None
        self.next_refresh_time: Optional[datetime] = None
        self.session_timeout: int = 8 * 60 * 60
        self._token: Optional[resources.Token] = None
        self._account_preferences: Optional[resources.AccountPreferences] = None
        self._account_balance: Optional[resources.Balance] = None
        self._last_balance_update_time: Optional[datetime] = None
        self._api_version = config.API_VERSION
        self._page_start_value = config.PAGE_START_VALUE
        self._minimum_limit_value = config.MINIMUM_LIMIT_VALUE

    @property
    def username(self) -> str:
        return self._username

    @property
    def api_version(self) -> str:
        return self._api_version

    @property
    def page_start_value(self) -> int:
        return self._page_start_value

    @property
    def minimum_limit_value(self) -> int:
        return self._minimum_limit_value

    @property
    def account_preferences(self) -> Optional[resources.AccountPreferences]:
        return self._account_preferences

    @property
    def environment(self) -> Environment:
        return self._environment

    @property
    def user_id(self) -> Optional[str]:
        return self.account_preferences.user_id if self.account_preferences else None

    @property
    def account_balance(self) -> Optional[resources.Balance]:
        return self._account_balance

    @property
    def last_balance_update_time(self) -> Optional[datetime]:
        return self._last_balance_update_time

    @property
    def logged_in(self) -> bool:
        try:
            return True if datetime.utcnow() < self.login_expiry_check else False
        except TypeError:
            return False

    def set_account_balance(self, account_balance: resources.Balance) -> None:
        """
        Sets the account balance for easy access on the client. This is only accurate upto the last time it was updated.
        :param account_balance: The account Balance resource
        :return: None
        """
        self._account_balance = account_balance
        self._last_balance_update_time = datetime.utcnow()
        logger.info(f"Updated user account balance")

    def set_account_preferences(
        self, account_preferences: resources.AccountPreferences
    ) -> None:
        """
        Sets the users account perferences to the client for easy access. User to determined user_id.
        :param account_preferences: An AccountPreferences resource. Contains user account data like user_id.
        :return: None
        """
        self._account_preferences = account_preferences
        if self.account_preferences.gamstop_result == "Y":
            raise exceptions.GamStopException()
        logger.info(f"Account preferences updated")

    def _update_client_session(self, session: Union[None, Session] = None) -> None:
        """
        Updates the client session with auth details
        :param session: a request Session
        :return: None
        """
        if session:
            self.session = session
            if session.auth is None:
                self.session.auth = (self._username, self._password)
        else:
            self.session = Session()
            self.session.auth = (self._username, self._password)
        self.session.headers.update({"X-API-KEY": self._api_key})
        self.login_expiry_check = datetime.utcnow()

        logger.debug(f"Account session updated")

    def _set_endpoint_uris(self, environment: Environment):
        """
        Sets the uri for which requests are sent to
        :param environment: enum Environment {STAGING,PRODUCTION}
        :return: None
        """
        if environment == Environment.PRODUCTION:
            try:
                assert self._personalised_production_url[-16:] == ".betconnect.com/"
                self.uri = self._personalised_production_url
            except KeyError:
                raise exceptions.InValidPersonalisedProductionURL(
                    url=self._personalised_production_url
                )
        elif environment == Environment.STAGING:
            self.uri = config.DEFAULT_STAGING_URL
        else:
            raise exceptions.UnknownBetConnectEnvironment(environment=environment)

    def process_login(self, token: str):
        """
        Processes the login, adding the token to the session header and setting login datetimes
        :param token: string token value supplied by BetConnect on login / token refresh
        :return: None
        """
        self.login_date = datetime.utcnow()
        self.login_expiry_check = datetime.utcnow() + timedelta(
            hours=config.LOGIN_FREQUENCY_CHECK_SECS
        )
        self.next_refresh_time = datetime.utcnow() + timedelta(
            minutes=config.CLIENT_TOKEN_REFRESH_FREQUENCY
        )
        self._token = token
        self.session.headers.update({"X-AUTH-TOKEN": token})
        logger.info(f"Account login successful")

    def process_logout(self):
        """
        Processes the logout, removing the token to the session header and setting login datetimes
        :return: None
        """
        self.login_date = None
        self.login_expiry_check = None
        self.next_refresh_time = None
        self._token = None
        try:
            self.session.headers.pop("X-AUTH-TOKEN")
        except KeyError:
            pass

        logger.info(f"Account logout successful")



from datetime import datetime
from typing import Tuple, Dict, Any
import pytest
from pytest_mock import MockerFixture
from requests import Response
from betconnect import resources
from betconnect.endpoints import Account
from betconnect import exceptions


class TestAccount:
    def test_get_user_preferences(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_get_user_preferences_response_response: Tuple[
            Response, Dict[str, Any], float
        ],
    ):
        get_user_preferences = mocker.patch(
            "betconnect.endpoints.account.Account._request",
            return_value=mock_get_user_preferences_response_response,
        )
        preferences = mock_account_endpoint.get_user_preferences()

        assert isinstance(preferences, resources.AccountPreferences)
        assert mock_account_endpoint.client.account_preferences == preferences
        get_user_preferences.assert_called()

    def test_get_balance(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_get_balance_response: Tuple[Response, Dict[str, Any], float],
    ):
        get_balance = mocker.patch(
            "betconnect.endpoints.account.Account._request",
            return_value=mock_get_balance_response,
        )
        balance = mock_account_endpoint.get_balance()
        assert isinstance(balance, resources.Balance)
        assert mock_account_endpoint.client.account_balance == balance
        get_balance.assert_called()

    # noinspection SpellCheckingInspection
    def test_login(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_login_response: Tuple[Response, Dict[str, Any], float],
        mock_login_failure_response: Tuple[Response, Dict[str, Any], float],
        mock_account_preferences_resource,
        mock_balance_resource,
    ):
        login_request = mocker.patch(
            "betconnect.endpoints.account.Account._post",
            return_value=mock_login_response,
        )
        get_user_preferences = mocker.patch(
            "betconnect.endpoints.account.Account.get_user_preferences",
            return_value=mock_account_preferences_resource,
        )
        get_balance = mocker.patch(
            "betconnect.endpoints.account.Account.get_balance",
            return_value=mock_balance_resource,
        )

        login = mock_account_endpoint.login()
        assert isinstance(login, resources.Login)
        login_request.assert_called_with(method_uri="api/v2/login")
        assert isinstance(mock_account_endpoint.client.login_date, datetime)
        assert isinstance(mock_account_endpoint.client.login_expiry_check, datetime)
        assert isinstance(mock_account_endpoint.client.next_refresh_time, datetime)
        assert (
            mock_account_endpoint.client._token
            == "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VySUQiOiJhNmExZWI5MS02MjE3LTRlMGMtYjc1OS1mYmNhYTJiN2E4YWMiLCJsYXN0X3JlZnJlc2giOjE2Mzk4NDk2NzguODAzODYzLCJleHAiOjE2Mzk4Nzg0Nzh9.SiH-iupnvhMXqFjIQuhnXS-YcbxKFXTwowRfytdlrBw"
        )
        assert (
            mock_account_endpoint.client.session.headers["X-AUTH-TOKEN"]
            == "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VySUQiOiJhNmExZWI5MS02MjE3LTRlMGMtYjc1OS1mYmNhYTJiN2E4YWMiLCJsYXN0X3JlZnJlc2giOjE2Mzk4NDk2NzguODAzODYzLCJleHAiOjE2Mzk4Nzg0Nzh9.SiH-iupnvhMXqFjIQuhnXS-YcbxKFXTwowRfytdlrBw"
        )

        assert isinstance(
            mock_account_endpoint.client.account_preferences,
            resources.AccountPreferences,
        )
        assert isinstance(
            mock_account_endpoint.client.account_balance, resources.Balance
        )

        login_request.assert_called()
        get_user_preferences.assert_called()
        get_balance.assert_called()

        login_request_failure = mocker.patch(
            "betconnect.endpoints.account.Account._post",
            return_value=mock_login_failure_response,
        )
        with pytest.raises(exceptions.FailedLogin):
            mock_account_endpoint.login()

        login_request_failure.assert_called()

    def test_logout(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_login_response: Tuple[Response, Dict[str, Any], float],
        mock_logout_response: Tuple[Response, Dict[str, Any], float],
    ):
        login_request = mocker.patch(
            "betconnect.endpoints.account.Account._post",
            return_value=mock_login_response,
        )
        mock_account_endpoint.login()
        logout_request = mocker.patch(
            "betconnect.endpoints.account.Account._post",
            return_value=mock_logout_response,
        )
        mock_account_endpoint.logout()
        assert mock_account_endpoint.client.login_date is None
        assert mock_account_endpoint.client.login_expiry_check is None
        assert mock_account_endpoint.client.next_refresh_time is None
        assert mock_account_endpoint.client._token is None
        assert "X-AUTH-TOKEN" not in mock_account_endpoint.client.session.headers
        login_request.assert_called()
        logout_request.assert_called()

    def test_refresh_session_token(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_refresh_session_token_response: Tuple[Response, Dict[str, Any], float],
    ):
        login_request = mocker.patch(
            "betconnect.endpoints.account.Account._post",
            return_value=mock_refresh_session_token_response,
        )
        status = mock_account_endpoint.refresh_session_token()
        assert isinstance(status, resources.Login)
        login_request.assert_called()

    def test__post(
        self,
        mocker: MockerFixture,
        mock_account_endpoint: Account,
        mock_login_response: Tuple[Response, Dict[str, Any], float],
    ):
        post = mocker.patch(
            "requests.sessions.Session.post", return_value=mock_login_response[0]
        )
        mock_account_endpoint._post(method_uri="test")
        post.assert_called()

    def test___call__(self, mocker: MockerFixture, mock_account_endpoint: Account):
        login = mocker.patch("betconnect.endpoints.account.Account.login")
        mock_account_endpoint()
        login.assert_called()



from betconnect import endpoints
import pytest
from betconnect import resources
from betconnect.apiclient import APIClient
from betconnect.baseclient import BaseClient
from betconnect.enums import Environment
from decouple import config
from requests import Response
from typing import Tuple, Dict, Any
from tests.utils import build_path, load_pickle, load_json
from pytest_mock import mocker


@pytest.fixture
def mock_base_client() -> BaseClient:
    return BaseClient(
        username="test",
        password="123",
        api_key="456",
        environment=Environment.STAGING,
        personalised_production_url=config("PRODUCTION_URI"),
    )


@pytest.fixture
def mock_api_client() -> APIClient:
    return APIClient(
        username="test",
        password="123",
        api_key="456",
        environment=Environment.STAGING,
        personalised_production_url=config("PRODUCTION_URI"),
    )


@pytest.fixture
def staging_api_client() -> APIClient:
    return APIClient(
        username=config("STAGING_BETCONNECT_USERNAME"),
        password=config("STAGING_BETCONNECT_PASSWORD"),
        api_key=config("STAGING_BETCONNECT_API_KEY"),
        environment=Environment.STAGING,
        personalised_production_url=config("PRODUCTION_URI"),
    )


@pytest.fixture
def staging_lay_api_client() -> APIClient:
    return APIClient(
        username=config("STAGING_BETCONNECT_LAY_USERNAME"),
        password=config("STAGING_BETCONNECT_LAY_PASSWORD"),
        api_key=config("STAGING_BETCONNECT_LAY_API_KEY"),
        environment=Environment.STAGING,
        personalised_production_url=config("PRODUCTION_URI"),
    )


@pytest.fixture
def mock_betting_endpoint(staging_api_client) -> endpoints.Betting:
    return endpoints.Betting(staging_api_client)


@pytest.fixture
def mock_account_endpoint(staging_api_client) -> endpoints.Account:
    return endpoints.Account(staging_api_client)


@pytest.fixture()
def mock_login_response_pkl() -> Response:
    return load_pickle(
        build_path(
            "resources/endpoints/account/mock_account_login_response_success.pkl"
        )
    )


@pytest.fixture()
def mock_login_response_json() -> Dict[str, Any]:
    return load_json(
        build_path(
            "resources/endpoints/account/mock_account_login_response_success.json"
        )
    )


@pytest.fixture()
def mock_login_response(
    mock_login_response_pkl: Response, mock_login_response_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_login_response_pkl, mock_login_response_json, 1.0


@pytest.fixture()
def mock_login_response_failure_pkl() -> Response:
    return load_pickle(
        build_path(
            "resources/endpoints/account/mock_account_login_response_failure.pkl"
        )
    )


@pytest.fixture()
def mock_login_response_failure_json() -> Dict[str, Any]:
    return load_json(
        build_path(
            "resources/endpoints/account/mock_account_login_response_failure.json"
        )
    )


@pytest.fixture()
def mock_login_failure_response(
    mock_login_response_failure_pkl: Response,
    mock_login_response_failure_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_login_response_failure_pkl, mock_login_response_failure_json, 1.0


@pytest.fixture()
def mock_logout_response_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/account/mock_account_logout_response.pkl")
    )


@pytest.fixture()
def mock_logout_response_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/account/mock_account_logout_response.json")
    )


@pytest.fixture()
def mock_logout_response(
    mock_logout_response_pkl: Response, mock_logout_response_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_logout_response_pkl, mock_logout_response_json, 1.0


@pytest.fixture()
def mock_active_bookmakers_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_bookmakers_response.pkl")
    )


@pytest.fixture()
def mock_active_bookmakers_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_bookmakers_response.json")
    )


@pytest.fixture()
def mock_active_bookmakers_response(
    mock_active_bookmakers_pkl: Response, mock_active_bookmakers_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_bookmakers_pkl, mock_active_bookmakers_json, 1.0


@pytest.fixture()
def mock_active_sports_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_sports_response.pkl")
    )


@pytest.fixture()
def mock_active_sports_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_sports_response.json")
    )


@pytest.fixture()
def mock_active_sports_response(
    mock_active_sports_pkl: Response, mock_active_sports_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_sports_pkl, mock_active_sports_json, 1.0


@pytest.fixture()
def mock_active_regions_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_regions_response.pkl")
    )


@pytest.fixture()
def mock_active_regions_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_regions_response.json")
    )


@pytest.fixture()
def mock_active_regions_response(
    mock_active_regions_pkl: Response, mock_active_regions_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_regions_pkl, mock_active_regions_json, 1.0


@pytest.fixture()
def mock_active_competitions_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_competitions_response.pkl")
    )


@pytest.fixture()
def mock_active_competitions_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_competitions_response.json")
    )


@pytest.fixture()
def mock_active_competitions_response(
    mock_active_competitions_pkl: Response,
    mock_active_competitions_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_competitions_pkl, mock_active_competitions_json, 1.0


@pytest.fixture()
def mock_active_fixtures_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_fixtures_response.pkl")
    )


@pytest.fixture()
def mock_active_fixtures_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_fixtures_response.json")
    )


@pytest.fixture()
def mock_active_fixtures_response(
    mock_active_fixtures_pkl: Response, mock_active_fixtures_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_fixtures_pkl, mock_active_fixtures_json, 1.0


@pytest.fixture()
def mock_active_market_types_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_market_types_response.pkl")
    )


@pytest.fixture()
def mock_active_market_types_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_market_types_response.json")
    )


@pytest.fixture()
def mock_active_market_types_response(
    mock_active_market_types_pkl: Response,
    mock_active_market_types_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_market_types_pkl, mock_active_market_types_json, 1.0


@pytest.fixture()
def mock_active_markets_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_markets_response.pkl")
    )


@pytest.fixture()
def mock_active_markets_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_markets_response.json")
    )


@pytest.fixture()
def mock_active_markets_response(
    mock_active_markets_pkl: Response, mock_active_markets_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_markets_pkl, mock_active_markets_json, 1.0


@pytest.fixture()
def mock_active_selections_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/active_selections_response.pkl")
    )


@pytest.fixture()
def mock_active_selections_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/active_selections_response.json")
    )


@pytest.fixture()
def mock_active_selections_response(
    mock_active_selections_pkl: Response, mock_active_selections_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_active_selections_pkl, mock_active_selections_json, 1.0


@pytest.fixture()
def mock_bet_history_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_history_response.pkl")
    )


@pytest.fixture()
def mock_bet_history_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_history_response.json")
    )


@pytest.fixture()
def mock_bet_history_response(
    mock_bet_history_pkl: Response, mock_bet_history_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_history_pkl, mock_bet_history_json, 1.0


@pytest.fixture()
def mock_get_balance_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/get_balance_response.pkl")
    )


@pytest.fixture()
def mock_get_balance_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/get_balance_response.json")
    )


@pytest.fixture()
def mock_get_balance_response(
    mock_get_balance_pkl: Response, mock_get_balance_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_get_balance_pkl, mock_get_balance_json, 1.0


@pytest.fixture()
def mock_my_bets_pkl() -> Response:
    return load_pickle(build_path("resources/endpoints/betting/my_bets_response.pkl"))


@pytest.fixture()
def mock_my_bets_json() -> Dict[str, Any]:
    return load_json(build_path("resources/endpoints/betting/my_bets_response.json"))


@pytest.fixture()
def mock_my_bets_response(
    mock_my_bets_pkl: Response, mock_my_bets_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_my_bets_pkl, mock_my_bets_json, 1.0


@pytest.fixture()
def mock_selections_for_market_line_market_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/selections_for_market_line_market.pkl")
    )


@pytest.fixture()
def mock_selections_for_market_line_market_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/selections_for_market_line_market.json")
    )


@pytest.fixture()
def mock_selections_for_market_line_market_response(
    mock_selections_for_market_line_market_pkl: Response,
    mock_selections_for_market_line_market_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return (
        mock_selections_for_market_line_market_pkl,
        mock_selections_for_market_line_market_json,
        1.0,
    )


@pytest.fixture()
def mock_prices_pkl() -> Response:
    return load_pickle(build_path("resources/endpoints/betting/prices_response.pkl"))


@pytest.fixture()
def mock_prices_json() -> Dict[str, Any]:
    return load_json(build_path("resources/endpoints/betting/prices_response.json"))


@pytest.fixture()
def mock_prices_response(
    mock_prices_pkl: Response, mock_prices_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_prices_pkl, mock_prices_json, 1.0


@pytest.fixture()
def mock_bet_request_create_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_request_create_response.pkl")
    )


@pytest.fixture()
def mock_bet_request_create_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_request_create_response.json")
    )


@pytest.fixture()
def mock_bet_request_create_response(
    mock_bet_request_create_pkl: Response, mock_bet_request_create_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_create_pkl, mock_bet_request_create_json, 1.0


@pytest.fixture()
def mock_bet_request_get_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_request_get_response.pkl")
    )


@pytest.fixture()
def mock_bet_request_get_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_request_get_response.json")
    )


@pytest.fixture()
def mock_bet_request_get_response(
    mock_bet_request_get_pkl: Response, mock_bet_request_get_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_get_pkl, mock_bet_request_get_json, 1.0


@pytest.fixture()
def mock_bet_request_match_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_request_match_response.pkl")
    )


@pytest.fixture()
def mock_bet_request_match_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_request_match_response.json")
    )


@pytest.fixture()
def mock_bet_request_match_response(
    mock_bet_request_match_pkl: Response, mock_bet_request_match_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_match_pkl, mock_bet_request_match_json, 1.0


@pytest.fixture()
def mock_bet_request_stop_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_request_stop_response.pkl")
    )


@pytest.fixture()
def mock_bet_request_stop_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_request_stop_response.json")
    )


@pytest.fixture()
def mock_bet_request_stop_response(
    mock_bet_request_stop_pkl: Response, mock_bet_request_stop_json: Dict[str, Any]
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_stop_pkl, mock_bet_request_stop_json, 1.0


@pytest.fixture()
def mock_bet_request_match_more_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/bet_request_match_more_response.pkl")
    )


@pytest.fixture()
def mock_bet_request_match_more_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/bet_request_match_more_response.json")
    )


@pytest.fixture()
def mock_bet_request_match_more_response(
    mock_bet_request_match_more_pkl: Response,
    mock_bet_request_match_more_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_match_more_pkl, mock_bet_request_match_more_json, 1.0


@pytest.fixture()
def mock_get_active_bet_requests_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/get_active_bet_requests_response.pkl")
    )


@pytest.fixture()
def mock_get_active_bet_requests_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/get_active_bet_requests_response.json")
    )


@pytest.fixture()
def mock_bet_get_active_bet_requests_response(
    mock_get_active_bet_requests_pkl: Response,
    mock_get_active_bet_requests_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_get_active_bet_requests_pkl, mock_get_active_bet_requests_json, 1.0


@pytest.fixture()
def mock_get_viewed_next_page_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/get_viewed_next_page_response.pkl")
    )


@pytest.fixture()
def mock_get_viewed_next_page_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/get_viewed_next_page_response.json")
    )


@pytest.fixture()
def mock_get_viewed_next_page_response(
    mock_get_viewed_next_page_pkl: Response,
    mock_get_viewed_next_page_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_get_viewed_next_page_pkl, mock_get_viewed_next_page_json, 1.0


@pytest.fixture()
def mock_selections_for_market_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/betting/selections_for_market_response.pkl")
    )


@pytest.fixture()
def mock_selections_for_market_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/betting/selections_for_market_response.json")
    )


@pytest.fixture()
def mock_selections_for_market_response(
    mock_selections_for_market_pkl: Response,
    mock_selections_for_market_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_selections_for_market_pkl, mock_selections_for_market_json, 1.0


"""
NEW ACCOUNT FIXTURES
"""


@pytest.fixture()
def mock_get_user_preferences_response_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/account/mock_get_user_preferences_response.pkl")
    )


@pytest.fixture()
def mock_get_user_preferences_response_json() -> Dict[str, Any]:
    return load_json(
        build_path(
            "resources/endpoints/account/mock_get_user_preferences_response.json"
        )
    )


@pytest.fixture()
def mock_get_user_preferences_response_response(
    mock_get_user_preferences_response_pkl: Response,
    mock_get_user_preferences_response_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return (
        mock_get_user_preferences_response_pkl,
        mock_get_user_preferences_response_json,
        1.0,
    )


@pytest.fixture()
def mock_login_resource() -> resources.Login:
    return resources.Login(message="Login Successful", data={"token": "12345"})


@pytest.fixture()
def mock_token_resource() -> resources.Token:
    return resources.Token(
        token="eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VySUQiOiJhNmExZWI5MS02MjE3LTRlMGMtYjc1OS1mYmNhYTJiN2E4YWMiLCJsYXN0X3JlZnJlc2giOjE2Mzk4NDk2NzguODAzODYzLCJleHAiOjE2Mzk4Nzg0Nzh9.SiH-iupnvhMXqFjIQuhnXS-YcbxKFXTwowRfytdlrBw"
    )


@pytest.fixture()
def mock_account_preferences_resource(
    mock_get_user_preferences_response_response: Tuple[Response, Dict[str, Any], float]
) -> resources.AccountPreferences:
    return resources.AccountPreferences(
        **mock_get_user_preferences_response_response[1]["data"]
    )


@pytest.fixture()
def mock_get_balance_response_pkl() -> Response:
    return load_pickle(
        build_path("resources/endpoints/account/mock_get_balance_response.pkl")
    )


@pytest.fixture()
def mock_get_balance_response_json() -> Dict[str, Any]:
    return load_json(
        build_path("resources/endpoints/account/mock_get_balance_response.json")
    )


@pytest.fixture()
def mock_get_balance_response(
    mock_get_balance_response_pkl: Response,
    mock_get_balance_response_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_get_balance_response_pkl, mock_get_balance_response_json, 1.0


@pytest.fixture()
def mock_balance_resource(
    mock_get_balance_response: Tuple[Response, Dict[str, Any], float]
) -> resources.Balance:
    return resources.Balance(**mock_get_balance_response[1]["data"])


@pytest.fixture()
def mock_bet_request_match_more_response(
    mock_bet_request_match_more_pkl: Response,
    mock_bet_request_match_more_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return mock_bet_request_match_more_pkl, mock_bet_request_match_more_json, 1.0


@pytest.fixture()
def mock_refresh_session_token_response_pkl() -> Response:
    return load_pickle(
        build_path(
            "resources/endpoints/account/mock_refresh_session_token_response.pkl"
        )
    )


@pytest.fixture()
def mock_refresh_session_token_response_json() -> Dict[str, Any]:
    return load_json(
        build_path(
            "resources/endpoints/account/mock_refresh_session_token_response.json"
        )
    )


@pytest.fixture()
def mock_refresh_session_token_response(
    mock_refresh_session_token_response_pkl: Response,
    mock_refresh_session_token_response_json: Dict[str, Any],
) -> Tuple[Response, Dict[str, Any], float]:
    return (
        mock_refresh_session_token_response_pkl,
        mock_refresh_session_token_response_json,
        1.0,
    )


@pytest.fixture()
def mock_active_bookmaker_resource() -> resources.ActiveBookmaker:
    return resources.ActiveBookmaker(
        name="10Bet", bookmaker_id=6122, order=99, active=1
    )


@pytest.fixture()
def mock_active_sport_resource() -> resources.ActiveSport:
    return resources.ActiveSport(
        id=21,
        sport_id=14,
        name="Horse Racing",
        display_name="Horse Racing",
        slug="horse-racing",
        order=1,
        active=1,
        rate=2.0,
    )


@pytest.fixture()
def mock_active_region_resource() -> resources.ActiveRegion:
    return resources.ActiveRegion(
        name="England", region_id=3795032, iso="england", order=2
    )


@pytest.fixture()
def mock_active_competition_resource() -> resources.ActiveCompetition:
    return resources.ActiveCompetition(
        name="Southwell",
        display_name="Southwell",
        competition_id=840,
        active=1,
        order=9999,
    )


@pytest.fixture()
def mock_active_market_type_resource() -> resources.ActiveMarketType:
    return resources.ActiveMarketType(market_type_id=6, name="WIN", active=1)


@pytest.fixture()
def mock_active_market_resource() -> resources.ActiveMarket:
    return resources.ActiveMarket(
        name="Race Winner",
        display_name="Race Winner",
        trading_status="Open",
        is_handicap="False",
        source_market_id="102347558",
        market_type_id=6,
        order=1,
        bet_types=["Win", "Each Way (1/4 odds 1-2-3)"],
    )


@pytest.fixture()
def mock_prices_bookmaker_resource() -> resources.PricesBookmaker:
    return resources.PricesBookmaker(id="123", name="Betfair")


@pytest.fixture()
def mock_price_resource() -> resources.Price:
    return resources.Price(
        price="10",
        numerator="10",
        denominator="1",
        bookmakers=[{"id": "123", "name": "Betfair"}],
    )


@pytest.fixture()
def mock_active_selection_resource() -> resources.ActiveSelection:
    return resources.ActiveSelection(
        name="Metal Man",
        trading_status="NonRunner",
        selection_id="308685953",
        ut="2021-12-17 15:49:20.068318",
        competitor="1095697",
    )


@pytest.fixture()
def mock_active_fixture_resource() -> resources.ActiveFixture:
    return resources.ActiveFixture(
        fixture_id="8757184",
        display_name="Kempton Park",
        start_date="2021-12-17",
        time="16:15",
        each_way_active="1",
    )


@pytest.fixture()
def mock_bet_request_resource() -> resources.BetRequest:
    return resources.BetRequest(
        sport_name="Horse Racing",
        sport_id="14",
        competition_name="Kempton Park",
        region_name="England",
        start_time_utc="2021-12-17 19:45:00+00:00",
        fixture_name="Kempton Park",
        market_name="Race Winner",
        selection_name="Starter For Ten",
        price={"decimal": "3.25", "fraction": {"numerator": "9", "denominator": "4"}},
        fixture_id=8757185,
        market_type_id=6,
        competitor="1207634",
        bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
        bet_type="Win",
        requested_stake=5.0,
        liability=11.25,
        locked_stake=0.0,
        backer_stats={
            "strike_rate": 0.33,
            "roi": -100.0,
            "bet_requests": "10-50",
            "recent_performance": "L,L,L,L,L,L,L,L",
        },
        others_viewing_bet=0,
        lockable=True,
    )


@pytest.fixture()
def mock_bet_request_create_resource() -> resources.BetRequestCreate:
    return resources.BetRequestCreate(
        bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
        created="2021-12-17 16:20:49.113386",
        debit_stake=5,
        debit_commission=0.1,
    )


@pytest.fixture()
def mock_active_bet_resource() -> resources.ActiveBet:
    return resources.ActiveBet(
        bet_request_id="45e1a024-523a-4b25-a33d-4466775d0aac",
        bet_type_name="Win",
        competition_name="Hamilton Park",
        created_at="2021-07-15 09:38:22",
        fill_percentage=0.0,
        fixture_id=8248855,
        fixture_name="Hamilton Park",
        fixture_start_date="2021-07-15 12:00:00",
        market_name="Race Winner",
        matched_stake=0.0,
        price=1.16,
        price_denominator=25,
        price_numerator=4,
        selection_name="Misty Ayr",
        sport_name="Horse Racing",
        stake=10.0,
        status_name="Active",
    )


@pytest.fixture()
def mock_bet_request_match_more_resource() -> resources.BetRequestMatch:
    return resources.BetRequestMatch(
        matched=True,
        available=True,
        bet_request_id="45e1a024-523a-4b25-a33d-4466775d0aac",
        bet_id="124",
        bet_status="Active",
        amount_matched=10.0,
    )


@pytest.fixture()
def mock_bet_history_resource() -> resources.BetHistory:
    return resources.BetHistory(
        bet_request_id="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
        bet_request_status="Active",
        bet_type_name="Win",
        competition_name="Kempton Park",
        fill_percentage=0.0,
        fixture_name="Kempton Park",
        fixture_start_date="2021-12-17 19:45:00",
        market_name="Race Winner",
        matched_stake=0.0,
        price=3.25,
        price_denominator=4,
        price_numerator=9,
        region_iso="england",
        region_name="England",
        selection_name="Starter For Ten",
        sport_name="Horse Racing",
        stake=5.0,
    )


@pytest.fixture()
def mock_active_bet_requests_resource(
    mock_active_bet_resource: resources.ActiveBet,
) -> resources.ActiveBetRequests:
    return resources.ActiveBetRequests(
        bets=[mock_active_bet_resource], bets_active=10, last_page=10, total_bets=20
    )


@pytest.fixture()
def mock_bet_history_request_resource(
    mock_bet_history_resource: resources.BetHistory,
) -> resources.ActiveBetRequests:
    return resources.ActiveBetRequests(
        bets=[mock_bet_history_resource], bets_active=10, last_page=10, total_bets=20
    )


@pytest.fixture()
def mock_selections_for_market_resource() -> resources.SelectionsForMarket:
    return resources.SelectionsForMarket(
        source_fixture_id="8757185",
        source_market_id="102347567",
        source_market_type_id="6",
        source_selection_id="308686039",
        trading_status="Trading",
        name="Starter For Ten",
        competitor_id="1207634",
        ut="2021-12-16 08:53:29.827063",
        order=0,
        max_price=3.25,
        prices=[
            {
                "price": "3.25",
                "numerator": "9",
                "denominator": "4",
                "bookmakers": [
                    {"id": "4", "name": "William Hill"},
                    {"id": "6002", "name": "Skybet"},
                ],
            },
            {
                "price": "3.20",
                "numerator": "11",
                "denominator": "5",
                "bookmakers": [
                    {"id": "3", "name": "Paddy Power"},
                    {"id": "6316", "name": "Betfair Sportsbook"},
                ],
            },
            {
                "price": "3.00",
                "numerator": "2",
                "denominator": "1",
                "bookmakers": [
                    {"id": "6006", "name": "Coral"},
                    {"id": "6001", "name": "BetVictor"},
                    {"id": "17", "name": "Ladbrokes"},
                ],
            },
        ],
    )


@pytest.fixture()
def mock_viewed_resource() -> resources.Viewed:
    return resources.Viewed(
        prev="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
        next="0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5589",
    )


@pytest.fixture()
def mock_bet_request_match_more_resource() -> resources.BetRequestMatchMore:
    return resources.BetRequestMatchMore(
        matched=True,
        available=True,
        viewed={
            "prev": "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5369",
            "next": "0f6efaa7-ffc3-45e9-8cf5-aa07e8ef5589",
        },
    )


@pytest.fixture()
def mock_bet_request_stop_resource() -> resources.BetRequestStop:
    return resources.BetRequestStop(pending=False)


@pytest.fixture()
def mock_my_active_bet_resource() -> resources.MyActiveBet:
    return resources.MyActiveBet(
        actioned_at="2021-07-15 09:35:26",
        bet_created=1,
        bet_request_id="a575ba8b-2264-4f14-8f59-9eeb805e5a37",
        bet_request_status_id=1,
        bet_request_user_id="a6a1eb91-6217-4e0c-b759-fbcaa2b7a8ac",
        bet_type_name="Win",
        competition_name="Hamilton Park",
        count_in_place=None,
        fill_percentage=0.0,
        fixture_name="Hamilton Park",
        fixture_start_date="2021-07-15 12:00:00",
        market_name="Race Winner",
        matched_stake=0,
        price={"decimal": "1.16", "fraction": {"denominator": 25, "numerator": 4}},
        profit=0,
        region_iso="scotland",
        region_name="Scotland",
        selection_name="Misty Ayr",
        sport_external_id=14,
        sport_name="Horse Racing",
        sport_slug="horse-racing",
        stake=1000,
        status_name="Active",
        status_slug="active",
    )


@pytest.fixture()
def mock_customer_order_ref_resource() -> resources.CustomerOrderRef:
    customer_order_ref_gen = resources.CustomerOrderRef.generate_random_order_ref()
    return resources.CustomerOrderRef(customer_order_ref=customer_order_ref_gen)


@pytest.fixture()
def mock_customer_strategy_ref_resource() -> resources.CustomerStrategyRef:
    customer_strategy_ref_value = "best_strategy"
    return resources.CustomerStrategyRef(
        customer_strategy_ref=customer_strategy_ref_value
    )



import uuid
from typing import Tuple, Dict, Any

import pytest
from pytest_mock import MockerFixture
from requests import Response
from betconnect import resources
from betconnect.apiclient import APIClient
from betconnect.endpoints import Betting
from uuid import UUID
from uuid import uuid4
from betconnect import enums


class TestBetting:
    def test___init__(self, mock_api_client: APIClient):
        configuration = Betting(mock_api_client)
        assert configuration.client == mock_api_client

    def test_active_bookmakers(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_bookmakers_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_bookmakers_response,
        )
        active_bookmakers = mock_betting_endpoint.active_bookmakers()
        assert isinstance(active_bookmakers, list)
        for s in active_bookmakers:
            assert isinstance(s, resources.ActiveBookmaker)
        request.assert_called()

    def test_active_competitions(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_competitions_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_competitions_response,
        )
        active_competitions = mock_betting_endpoint.active_competitions(
            sport_id=14, region_id=3795032
        )
        assert isinstance(active_competitions, list)
        for r in active_competitions:
            assert isinstance(r, resources.ActiveCompetition)
        request.assert_called()

    def test_active_fixtures(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_fixtures_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_fixtures_response,
        )
        active_fixtures = mock_betting_endpoint.active_fixtures(sport_id=14)
        assert isinstance(active_fixtures, list)
        for r in active_fixtures:
            assert isinstance(r, resources.ActiveFixture)

        active_fixtures = mock_betting_endpoint.active_fixtures(
            sport_id=14, region_id=3795032
        )
        assert isinstance(active_fixtures, list)
        for r in active_fixtures:
            assert isinstance(r, resources.ActiveFixture)

        active_fixtures = mock_betting_endpoint.active_fixtures(
            sport_id=14, region_id=3795032, competition_id=828
        )
        assert isinstance(active_fixtures, list)
        for r in active_fixtures:
            assert isinstance(r, resources.ActiveFixture)
        request.assert_called()

    def test_active_market_types(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_market_types_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_market_types_response,
        )
        active_market_types = mock_betting_endpoint.active_market_types(sport_id=14)
        assert isinstance(active_market_types, list)
        for r in active_market_types:
            assert isinstance(r, resources.ActiveMarketType)
        request.assert_called()

    def test_active_markets(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_markets_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_markets_response,
        )
        active_markets = mock_betting_endpoint.active_markets(fixture_id=8573302)
        assert isinstance(active_markets, list)
        for r in active_markets:
            assert isinstance(r, resources.ActiveMarket)

        active_markets = mock_betting_endpoint.active_markets(
            fixture_id=8573302, grouped=True
        )
        assert isinstance(active_markets, list)
        for r in active_markets:
            assert isinstance(r, resources.ActiveMarket)

        request.assert_called()

    def test_active_regions(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_regions_response: Tuple[Response, Dict[str, Any], float],
    ):

        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_regions_response,
        )

        active_regions = mock_betting_endpoint.active_regions(sport_id=14)
        assert isinstance(active_regions, list)
        for r in active_regions:
            assert isinstance(r, resources.ActiveRegion)
        request.assert_called()

    def test_active_sports(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_sports_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_sports_response,
        )

        active_sports = mock_betting_endpoint.active_sports()
        assert isinstance(active_sports, list)
        for s in active_sports:
            assert isinstance(s, resources.ActiveSport)
        request.assert_called()

        active_sports = mock_betting_endpoint.active_sports(with_bets=True)
        assert isinstance(active_sports, list)
        for s in active_sports:
            assert isinstance(s, resources.ActiveSport)
        request.assert_called()

    def test_active_selections(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_active_selections_response: Tuple[Response, Dict[str, Any], float],
    ):

        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_active_selections_response,
        )
        active_selections = mock_betting_endpoint.active_selections(
            fixture_id=8573302, market_type_id=6
        )
        assert isinstance(active_selections, list)
        for r in active_selections:
            assert isinstance(r, resources.ActiveSelection)

        active_selections = mock_betting_endpoint.active_selections(
            fixture_id=8234079, market_type_id=6
        )
        assert isinstance(active_selections, list)
        for r in active_selections:
            assert isinstance(r, resources.ActiveSelection)
        request.assert_called()

    def test_bet_history(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_bet_history_response: Tuple[Response, Dict[str, Any], float],
    ):

        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_bet_history_response,
        )
        bet_history = mock_betting_endpoint.bet_history(
            status=enums.BetStatus.MATCHED, side=enums.BetSide.BACK
        )
        assert isinstance(bet_history, resources.BetHistoryRequest)
        request.assert_called()

    def test_my_bets(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_my_bets_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_my_bets_response,
        )

        # Get my bets
        my_bets = mock_betting_endpoint.my_bets(
            side=enums.BetSide.BACK,
            user_id="12345",
            status=enums.BetRequestStatus.SETTLED,
            limit=100,
            page=0,
            get_all="get_all",
        )
        assert isinstance(my_bets, resources.MyBetsBetRequests)
        request.assert_called()

    def test_prices(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_prices_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_prices_response,
        )
        prices = mock_betting_endpoint.prices(
            fixture_id=8573295, market_type_id=6, competitor="1247097"
        )
        assert isinstance(prices, list)
        for p in prices:
            assert isinstance(p, resources.Price)
        request.assert_called()

    def test_bet_request_get(
        self,
        mocker: MockerFixture,
        staging_lay_api_client,
        mock_betting_endpoint: Betting,
        mock_bet_request_get_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._post",
            return_value=mock_bet_request_get_response,
        )
        bet_request = mock_betting_endpoint.bet_request_get(
            request_filter=resources.GetBetRequestFilter(
                sport_id=14, bet_request_id="712d7387-c059-4d58-be66-12eccdeed384"
            )
        )
        assert isinstance(bet_request, resources.BetRequest)
        request.assert_called()

    def test_bet_request_create(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_bet_request_create_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._post",
            return_value=mock_bet_request_create_response,
        )
        bet_request = mock_betting_endpoint.bet_request_create(
            request_filter=resources.CreateBetRequestFilter(
                fixture_id=8573295,
                market_type_id=6,
                competitor="1247097",
                price=2.63,
                stake=100,
                bet_type="Win",
            )
        )
        assert isinstance(bet_request, resources.BetRequestCreate)
        request.assert_called()

    def test_selections_for_market(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_selections_for_market_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_selections_for_market_response,
        )
        seletions = mock_betting_endpoint.selections_for_market(
            fixture_id=8763863, market_type_id=6, top_price_only=False
        )
        assert isinstance(seletions, list)
        for seletion in seletions:
            assert isinstance(seletion, resources.SelectionsForMarket)
        request.assert_called()

    def test_bet_request_match(
        self,
        mocker: MockerFixture,
        staging_lay_api_client: APIClient,
        mock_betting_endpoint: Betting,
        mock_bet_request_match_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._patch",
            return_value=mock_bet_request_match_response,
        )
        mock_betting_endpoint.client = staging_lay_api_client
        bet_request_match = mock_betting_endpoint.bet_request_match(
            bet_request_id=UUID("d05215ce-d46f-40fd-8fc6-232dedcfdce4"),
            accepted_stake=10,
        )
        assert isinstance(bet_request_match, resources.BetRequestMatch)
        request.assert_called()

    def test_bet_request_match_more(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint,
        staging_lay_api_client,
        mock_bet_request_match_more_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._patch",
            return_value=mock_bet_request_match_more_response,
        )
        mock_betting_endpoint.client = staging_lay_api_client
        bet_request = mock_betting_endpoint.bet_request_match_more(
            bet_request_id=UUID("d05215ce-d46f-40fd-8fc6-232dedcfdce4"),
            requested_stake=50,
        )
        assert isinstance(bet_request, resources.BetRequestMatchMore)
        request.assert_called()

    def test_bet_request_stop(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_bet_request_stop_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._post",
            return_value=mock_bet_request_stop_response,
        )
        bet_request_stop = mock_betting_endpoint.bet_request_stop(
            bet_request_id=UUID("d05215ce-d46f-40fd-8fc6-232dedcfdce4")
        )
        assert isinstance(bet_request_stop, resources.ResponseMessage)
        request.assert_called()

    def test_get_active_bet_requests(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_bet_get_active_bet_requests_response: Tuple[
            Response, Dict[str, Any], float
        ],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_bet_get_active_bet_requests_response,
        )
        active_bets = mock_betting_endpoint.get_active_bet_requests()
        assert isinstance(active_bets, resources.ActiveBetRequests)
        request.assert_called()

    def test_get_viewed_next_page(
        self,
        mocker: MockerFixture,
        mock_betting_endpoint: Betting,
        mock_get_viewed_next_page_response: Tuple[Response, Dict[str, Any], float],
    ):
        request = mocker.patch(
            "betconnect.endpoints.baseendpoint.BaseEndpoint._request",
            return_value=mock_get_viewed_next_page_response,
        )
        next_page = mock_betting_endpoint.get_viewed_next_page(
            bet_request_id=uuid4(), sport_id=14
        )
        assert isinstance(next_page, resources.Viewed)
        request.assert_called()

    def test__is_line_market(
        self,
        mock_betting_endpoint: Betting,
        mock_selections_for_market_line_market_response: Tuple[
            Response, Dict[str, Any], float
        ],
        mock_selections_for_market_response: Tuple[Response, Dict[str, Any], float],
    ):
        assert mock_betting_endpoint._is_line_market(
            mock_selections_for_market_line_market_response[0],
            mock_selections_for_market_line_market_response[1],
        )
        assert (
            mock_betting_endpoint._is_line_market(
                mock_selections_for_market_response[0],
                mock_selections_for_market_response[1],
            )
            is False
        )

    def test_lock_bet(self, mock_betting_endpoint):
        with pytest.raises(NotImplementedError):
            mock_betting_endpoint.lock_bet(uuid.uuid4(), 1, 1)



from .baseresource import BaseResource
from pydantic import Field, validator
from typing import Union, List, Optional
import logging
from betconnect import resources
from betconnect.utils import is_valid_uuid
from betconnect import config, exceptions
from uuid import UUID

logger = logging.getLogger(__name__)


class Filter(BaseResource):
    def generate_request_data(self, *args, **kwargs) -> dict:
        raise NotImplementedError

    def validate_stake(v: float) -> float:
        if v < config.SITE_MINIMUM_STAKE_SIZE:
            raise exceptions.BetRequestIDStakeSizeException(stake_size=v)
        else:
            return v


class GetBetRequestFilter(Filter):
    sport_id: Optional[int] = Field(default=None)
    bookmakers: Optional[List[int]] = Field(default=[])
    horse_racing_regions: List[int] = Field(default=[], alias="horseRacingRegions")
    min_odds: Optional[float] = Field(default=1.01)
    max_odds: Optional[float] = Field(default=1000)
    accept_each_way: Optional[bool] = Field(default=True)
    bet_request_id: Optional[UUID] = Field(default=None)

    # noinspection PyMethodParameters
    @validator("min_odds", pre=True)
    def validate_min_odds(cls, v: Optional[float]) -> Union[float, None]:
        if v:
            if v < 1.01:
                raise exceptions.MinOddException(min_odds=v)
        return v

    # noinspection PyMethodParameters
    @validator("bet_request_id", pre=True)
    def validate_bet_request_id(cls, v: Union[UUID, Optional[str]]) -> Optional[UUID]:
        if isinstance(v, UUID):
            return v
        elif v:
            if is_valid_uuid(v):
                return UUID(v)
            else:
                raise exceptions.BetRequestIDFormatException(bet_request_id=v)

    def generate_request_data(self, *args, **kwargs) -> dict:
        data = self.dict(exclude=kwargs["exclude"])
        if "bet_request_id" in data:
            data["bet_request_id"] = str(data["bet_request_id"])
        return data


class CreateBetRequestBySelectionFilter(Filter):
    selection_id: int
    price: float
    stake: float
    handicap: Optional[float] = Field(default=None)
    bet_type: str
    customer_strategy_ref: Optional[resources.CustomerStrategyRef] = Field(default=None)
    customer_order_ref: Optional[resources.CustomerOrderRef] = Field(default=None)

    # inherited size validator
    _size = validator("stake", allow_reuse=True)(Filter.validate_stake)

    @validator("customer_strategy_ref", pre=True)
    def parse_customer_strategy_ref(
        cls, v: Union[str, resources.CustomerStrategyRef]
    ) -> resources.CustomerStrategyRef:
        if isinstance(v, resources.CustomerStrategyRef):
            return v
        return resources.CustomerStrategyRef.create_customer_strategy_ref(
            customer_strategy_ref=v
        )

    @validator("customer_order_ref", pre=True)
    def parse_customer_order_ref(
        cls, v: Union[str, resources.CustomerOrderRef]
    ) -> resources.CustomerOrderRef:
        if isinstance(v, resources.CustomerOrderRef):
            return v
        return resources.CustomerOrderRef.create_customer_order_ref(
            customer_order_ref=v
        )

    def generate_request_data(self, *args, **kwargs) -> dict:
        data = self.dict(exclude_none=kwargs["exclude_none"])
        if "customer_order_ref" in data:
            data["customer_order_ref"] = data["customer_order_ref"][
                "customer_order_ref"
            ]
        if "customer_strategy_ref" in data:
            data["customer_strategy_ref"] = data["customer_strategy_ref"][
                "customer_strategy_ref"
            ]

        return data


class CreateBetRequestByCompetitorFilter(Filter):
    fixture_id: int
    market_type_id: int
    competitor: str
    price: float
    stake: float
    handicap: Optional[float] = Field(default=None)
    bet_type: str
    customer_strategy_ref: Optional[resources.CustomerStrategyRef] = Field(default=None)
    customer_order_ref: Optional[resources.CustomerOrderRef] = Field(default=None)

    # inherited size validator
    _size = validator("stake", allow_reuse=True)(Filter.validate_stake)

    @validator("customer_strategy_ref", pre=True)
    def parse_customer_strategy_ref(
        cls, v: Union[str, resources.CustomerStrategyRef]
    ) -> resources.CustomerStrategyRef:
        if isinstance(v, resources.CustomerStrategyRef):
            return v
        return resources.CustomerStrategyRef.create_customer_strategy_ref(
            customer_strategy_ref=v
        )

    @validator("customer_order_ref", pre=True)
    def parse_customer_order_ref(
        cls, v: Union[str, resources.CustomerOrderRef]
    ) -> resources.CustomerOrderRef:
        if isinstance(v, resources.CustomerOrderRef):
            return v
        return resources.CustomerOrderRef.create_customer_order_ref(
            customer_order_ref=v
        )

    def generate_request_data(self, *args, **kwargs) -> dict:
        data = self.dict(exclude_none=kwargs["exclude_none"])
        if "customer_order_ref" in data:
            data["customer_order_ref"] = data["customer_order_ref"][
                "customer_order_ref"
            ]
        if "customer_strategy_ref" in data:
            data["customer_strategy_ref"] = data["customer_strategy_ref"][
                "customer_strategy_ref"
            ]

        return data


# alias for backward compatibility
CreateBetRequestFilter = CreateBetRequestByCompetitorFilter



from .baseresource import BaseResource


class ResponseMessage(BaseResource):
    message: str
    request_url: str
    status_code: int


class BaseRequestException(BaseResource):
    message: str
    request_url: str
    status_code: int



from betconnect import exceptions
import pytest
from betconnect import enums


class TestExceptions:
    def test_invalid_personalised_production_url(self):
        exception = exceptions.InValidPersonalisedProductionURL(url="test.com")
        assert exception.url == "test.com"

        with pytest.raises(exceptions.InValidPersonalisedProductionURL):
            raise exceptions.InValidPersonalisedProductionURL(url="test.com")

    def test_unknown_betconnect_environment(self):
        exception = exceptions.UnknownBetConnectEnvironment(
            environment=enums.Environment.PRODUCTION
        )
        assert exception.environment == enums.Environment.PRODUCTION

        with pytest.raises(exceptions.UnknownBetConnectEnvironment):
            raise exceptions.UnknownBetConnectEnvironment(
                environment=enums.Environment.PRODUCTION
            )

    def test_login_missing_token_in_response(self):
        exception = exceptions.LoginMissingTokenInResponse()
        assert str(exception) == f"Missing login token in response data"

        with pytest.raises(exceptions.LoginMissingTokenInResponse):
            raise exceptions.LoginMissingTokenInResponse()

    def test_failed_login(self):
        exception = exceptions.FailedLogin(username="test_username")
        assert exception.username == "test_username"

        with pytest.raises(exceptions.FailedLogin):
            raise exceptions.FailedLogin("test_username")

    def test_unexpected_response_status_code(self):
        exception = exceptions.UnexpectedResponseStatusCode(
            status_code=12345, url="betconnect.com"
        )
        assert exception.status_code == 12345
        assert exception.url == "betconnect.com"

        with pytest.raises(exceptions.UnexpectedResponseStatusCode):
            raise exceptions.UnexpectedResponseStatusCode(
                status_code=12345, url="betconnect.com"
            )

    def test_gamstop_exception(self):
        exception = exceptions.GamStopException()
        assert (
            str(exception)
            == "API access if strictly forbidden for users with a Gamstop ban against their account."
        )

        with pytest.raises(exceptions.GamStopException):
            raise exceptions.GamStopException()

    def test_missing_user_preferences(self):
        exception = exceptions.MissingUserPerferences()
        assert (
            str(exception)
            == "Unable to get user_id via get_user_preferences. Please supply a value or speak to your account manager"
        )

        with pytest.raises(exceptions.MissingUserPerferences):
            raise exceptions.MissingUserPerferences()

    def test_min_odds(self):
        exception = exceptions.MinOddException(min_odds=0)
        assert exception.min_odds == 0

        with pytest.raises(exceptions.MinOddException):
            raise exceptions.MinOddException(min_odds=0)

    def test_bet_request_stale_size(self):
        exception = exceptions.BetRequestIDStakeSizeException(stake_size=9)
        assert exception.stake_size == 9

        with pytest.raises(exceptions.BetRequestIDStakeSizeException):
            raise exceptions.BetRequestIDStakeSizeException(stake_size=9)

    def test_bet_request_invalid_customer_order_ref(self):
        exception = exceptions.BetRequestInvalidCustomerOrderRefFormatException(
            customer_order_ref="1234"
        )
        assert exception.customer_order_ref == "1234"

        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            raise exceptions.BetRequestInvalidCustomerOrderRefFormatException(
                customer_order_ref="1234"
            )

    def test_bet_request_invalid_customer_strategy_ref(self):
        exception = exceptions.BetRequestInvalidCustomerStrategyRefFormatException(
            customer_strategy_ref=""
        )
        assert exception.customer_strategy_ref == ""

        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            raise exceptions.BetRequestInvalidCustomerStrategyRefFormatException(
                customer_strategy_ref=""
            )

    def test_api_error(self):
        exceptions.APIError(response=None, uri="betconnect.com")
        with pytest.raises(exceptions.APIError):
            raise exceptions.APIError(response=None, uri="betconnect.com")



import uuid
from datetime import datetime, time
from typing import List, Optional
from pydantic import Field, validator
from .baseresource import BaseResource
from uuid import UUID
import logging
from betconnect import config
from betconnect import utils
from betconnect import exceptions

logger = logging.getLogger(__name__)


class CustomerOrderRef(BaseResource):
    customer_order_ref: Optional[str] = Field(default=None)

    @staticmethod
    def generate_random_order_ref() -> str:
        return str(uuid.uuid4())

    @staticmethod
    def is_valid_customer_order_ref(customer_order_ref: str) -> bool:
        if (len(customer_order_ref) < config.MIN_CUSTOMER_ORDER_REF_LENGTH) or (
            len(customer_order_ref) > config.MAX_CUSTOMER_ORDER_REF_LENGTH
        ):
            return False
        return True

    @classmethod
    def create_customer_order_ref(cls, customer_order_ref: str):
        if cls.is_valid_customer_order_ref(customer_order_ref=customer_order_ref):
            return cls(customer_order_ref=customer_order_ref)
        raise exceptions.BetRequestInvalidCustomerOrderRefFormatException(
            customer_order_ref=customer_order_ref
        )

    @validator("customer_order_ref", pre=True)
    def validate_customer_order_ref(cls, v: str) -> Optional[str]:
        if v:
            assert isinstance(v, str)
            if " " in v:
                v = v.replace(" ", "")
                logger.warning(
                    f"customer_order_ref contains spaces. These have been removed"
                )
            if cls.is_valid_customer_order_ref(v):
                return v
            else:
                raise exceptions.BetRequestInvalidCustomerOrderRefFormatException(
                    customer_order_ref=v
                )

    def __repr__(self) -> str:
        return f"Customer order ref: {self.customer_order_ref}"

    def __str__(self) -> str:
        return self.customer_order_ref


class CustomerStrategyRef(BaseResource):
    customer_strategy_ref: Optional[str] = Field(default=None)

    @staticmethod
    def is_valid_customer_strategy_ref(customer_strategy_ref: str) -> bool:
        if (len(customer_strategy_ref) < config.MIN_CUSTOMER_STRATEGY_REF_LENGTH) or (
            len(customer_strategy_ref) > config.MAX_CUSTOMER_STRATEGY_REF_LENGTH
        ):
            return False
        return True

    @classmethod
    def create_customer_strategy_ref(
        cls, customer_strategy_ref: str, hash_value: bool = False
    ):
        if cls.is_valid_customer_strategy_ref(
            customer_strategy_ref=customer_strategy_ref
        ):
            if hash_value:
                return cls(
                    customer_strategy_ref=utils.create_cheap_hash(
                        txt=customer_strategy_ref, length=15
                    )
                )
            else:
                return cls(customer_strategy_ref=customer_strategy_ref)
        raise exceptions.BetRequestInvalidCustomerStrategyRefFormatException(
            f"customer_strategy_ref ({customer_strategy_ref}) should be between 1-15 characters. Supplied length = {len(customer_strategy_ref)}"
        )

    # noinspection PyMethodParameters
    @validator("customer_strategy_ref", pre=True)
    def validate_customer_strategy_ref(cls, v: str) -> str:
        if v:
            assert isinstance(v, str)
            if " " in v:
                v = v.replace(" ", "")
                logger.warning(
                    f"customer_strategy_ref contains spaces. These have been removed"
                )
            if cls.is_valid_customer_strategy_ref(customer_strategy_ref=v):
                return v

            raise exceptions.BetRequestInvalidCustomerStrategyRefFormatException(
                customer_strategy_ref=v
            )

    def __str__(self) -> str:
        return self.customer_strategy_ref

    def __repr__(self) -> str:
        return f"Customer strategy ref: {self.customer_strategy_ref}"


class ActiveBookmaker(BaseResource):
    name: str
    bookmaker_id: int
    order: int
    active: int

    def __repr__(self) -> str:
        return f"Bookmaker: {self.name} ({self.bookmaker_id}), Active: {self.active}"


class ActiveSport(BaseResource):
    id: int
    sport_id: int
    name: str
    display_name: str
    slug: str
    order: int
    active: int
    rate: float
    bets_available: Optional[int] = Field(default=None)

    def __repr__(self) -> str:
        return f"Sport: {self.display_name} ({self.id})"


class ActiveRegion(BaseResource):
    name: str
    region_id: int
    iso: Optional[str] = Field(default=None)
    order: int

    def __repr__(self) -> str:
        return "Region: {}({})".format(self.name, self.region_id)


class ActiveCompetition(BaseResource):
    name: str
    display_name: str
    competition_id: int
    active: int
    order: int

    # waiting on sport_id: int
    # waiting on region_id: int

    def __repr__(self) -> str:
        return f"Competition: {self.name}({self.competition_id})"


class ActiveMarketType(BaseResource):
    market_type_id: int
    name: str
    active: int

    def __repr__(self) -> str:
        return f"Market Type: {self.name}({self.market_type_id})"


class ActiveMarket(BaseResource):
    name: str
    display_name: str
    trading_status: str
    is_handicap: str
    source_market_id: str
    market_type_id: int
    order: int
    handicap: Optional[str] = Field(default=None)
    bet_types: List[str] = []

    # waiting on fixture_id: int

    def __repr__(self) -> str:
        return f"Market: {self.name} {self.trading_status}"


class Balance(BaseResource):
    balance: int

    @property
    def balance_uk_pounds(self) -> float:
        try:
            return round(self.balance / 100, 3)
        except ZeroDivisionError:
            return 0

    def __repr__(self) -> str:
        return f"Balance: {self.balance}"


class PricesBookmaker(BaseResource):
    id: str
    name: str

    def __repr__(self) -> str:
        return f"Bookie Name: {self.name} ({self.id})"


class Price(BaseResource):
    price: str
    numerator: int
    denominator: int
    bookmakers: List[PricesBookmaker] = Field(default=[])

    @classmethod
    def create_from_dict(cls, d):
        return [cls.parse_obj(data) for data in d["prices"]]

    def __repr__(self) -> str:
        return f"Price: {self.price}, Bookmakers #:{len(self.bookmakers)}"

    def __eq__(self, other) -> bool:
        if isinstance(other, Price):
            return other.price == self.price
        return False


class ActiveSelection(BaseResource):
    name: str
    trading_status: str
    selection_id: str
    ut: datetime
    competitor: str

    # waiting on fixture_id: int

    # noinspection PyMethodParameters
    @validator("ut", pre=True)
    def date_parser(cls, v):
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __eq__(self, other) -> bool:
        if isinstance(other, ActiveSelection):
            return other.selection_id == self.selection_id
        return False

    def __repr__(self) -> str:
        return f"Selection: {self.name} ({self.selection_id})"


class ActiveFixture(BaseResource):
    fixture_id: int
    display_name: str
    start_date: datetime = Field(alias="startdate")
    time: time
    each_way_active: Optional[str] = Field(default=None)

    # waiting on sport_id: int ?
    # waiting on region_id: int ?
    # waiting on competition_id: int ?

    @property
    def start_date_time(self) -> datetime:
        return self.start_date.replace(
            hour=self.time.hour, minute=self.time.minute, second=self.time.second
        )

    # noinspection PyMethodParameters
    @validator("start_date", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __repr__(self) -> str:
        return f"Fixture: {self.display_name}({self.fixture_id}) {self.start_date}"


class BetRequest(BaseResource):
    sport_name: str
    sport_id: str
    competition_name: str
    region_name: str
    start_time_utc: datetime
    fixture_name: str
    market_name: str
    selection_name: str
    price: Price
    fixture_id: int
    market_type_id: int
    competitor: str
    bet_request_id: UUID
    bet_type: Optional[str] = Field(default=None)
    requested_stake: float
    liability: float
    locked_stake: float
    others_viewing_bet: int
    lockable: bool

    # noinspection PyMethodParameters
    @validator("price", pre=True)
    def price_parser(cls, v) -> Price:
        if isinstance(v, dict):
            return Price(
                price=v["decimal"],
                numerator=v["fraction"]["numerator"],
                denominator=v["fraction"]["denominator"],
            )
        else:
            raise TypeError(f"Expected value of dict")

    # noinspection PyMethodParameters
    @validator("start_time_utc", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __hash__(self):
        return hash(
            f"{self.fixture_name}-{self.selection_name}-{self.start_time_utc.isoformat()}"
        )


class BetRequestCreate(BaseResource):
    bet_request_id: UUID
    created: datetime
    debit_stake: float
    debit_commission: float

    # noinspection PyMethodParameters
    @validator("created", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Debited Stake: {self.debit_stake}"


class BetRequestMatch(BaseResource):
    matched: bool
    available: bool
    bet_request_id: Optional[UUID] = Field(default=None)
    bet_id: Optional[str] = Field(default=None)
    bet_status: Optional[str] = Field(default=None)
    amount_matched: Optional[float] = Field(default=None)
    viewed: Optional[dict] = Field(default=None)

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Matched: {self.matched}, Available: {self.available}"


class BetHistory(BaseResource):
    bet_request_id: UUID
    bet_request_status: str
    bet_type_name: Optional[str] = Field(default=None)
    competition_name: str
    create_at: Optional[datetime] = Field(default=None)
    each_way_factor: Optional[str] = Field(default=None)
    fill_percentage: float
    fixture_name: str
    fixture_start_date: datetime = Field(alias="fixture_startdate")
    handicap: Optional[float] = Field(default=None)
    market_name: str
    matched_stake: float
    price: float
    price_denominator: int
    price_numerator: int
    region_iso: Optional[str] = Field(default=None)
    region_name: str
    selection_name: str
    sport_name: str
    stake: float

    # noinspection PyMethodParameters
    @validator("create_at", "fixture_start_date", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Competition: {self.competition_name}, Selection: {self.selection_name},Price: {self.price}, Stake: {self.stake}"


class MyActiveBet(BaseResource):
    actioned_at: datetime
    adjustment: Optional[str] = Field(default=None)  # TODO check type
    adjustment_type: Optional[str] = Field(default=None)  # TODO check type
    bet_created: int
    bet_request_id: UUID
    bet_request_status_id: int
    bet_request_user_id: UUID
    bet_type_name: Optional[str] = Field(default=None)
    competition_name: str
    count_in_place: Optional[int] = Field(default=None)  # TODO check type
    customer_order_ref: Optional[CustomerOrderRef] = Field(default=None)
    customer_strategy_ref: Optional[CustomerStrategyRef] = Field(default=None)
    each_way_factor: Optional[int] = Field(default=None)
    fill_percentage: float
    fixture_name: str
    fixture_start_date: datetime = Field(alias="fixture_startdate")
    handicap: Optional[str] = Field(default=None)
    market_name: str
    matched_stake: int
    price: Price
    profit: int
    profit_loss: Optional[int] = Field(default=None)
    region_iso: Optional[str] = Field(default=None)
    region_name: str
    result_type_name: Optional[str] = Field(default=None)
    selection_name: str
    sport_external_id: int
    sport_name: str
    sport_slug: str
    stake: int
    status_name: str
    status_slug: str
    sub_account_id: Optional[str] = Field(default=None)  # TODO check type

    # noinspection PyMethodParameters
    @validator("actioned_at", "fixture_start_date", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    # noinspection PyMethodParameters
    @validator("customer_order_ref", pre=True)
    def parse_customer_order_ref(cls, v) -> Optional[CustomerOrderRef]:
        if v:
            return CustomerOrderRef(customer_order_ref=v)

    # noinspection PyMethodParameters
    @validator("customer_strategy_ref", pre=True)
    def parse_customer_strategy_ref(cls, v) -> Optional[CustomerStrategyRef]:
        if v:
            return CustomerStrategyRef(customer_strategy_ref=v)

    # noinspection PyMethodParameters
    @validator("price", pre=True)
    def price_parser(cls, v) -> Price:
        if isinstance(v, dict):
            return Price(
                price=v["decimal"],
                numerator=v["fraction"]["numerator"],
                denominator=v["fraction"]["denominator"],
            )
        else:
            raise TypeError(f"Expected value of dict")

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Competition: {self.competition_name}, Selection: {self.selection_name},Price: {self.price}, Stake: {self.stake}, Profit: {self.profit}"


class MyActiveLayBet(BaseResource):
    actioned_at: datetime
    adjustment: Optional[str] = Field(default=None)  # TODO check type
    adjustment_type: Optional[str] = Field(default=None)  # TODO check type
    bet_created: int
    bet_request_id: UUID
    bet_request_status_id: int
    bet_type_name: Optional[str] = Field(default=None)
    competition_name: str
    count_in_place: Optional[int] = Field(default=None)  # TODO check type
    customer_order_ref: Optional[CustomerOrderRef] = Field(default=None)
    customer_strategy_ref: Optional[CustomerStrategyRef] = Field(default=None)
    each_way_factor: Optional[int] = Field(default=None)
    fixture_name: str
    fixture_start_date: datetime = Field(alias="fixture_startdate")
    handicap: Optional[str] = Field(default=None)
    liability: int
    market_name: str
    price: Price
    profit_loss: Optional[int] = Field(default=None)
    region_iso: Optional[str] = Field(default=None)
    region_name: str
    result_type_name: Optional[str] = Field(default=None)
    selection_name: str
    sport_external_id: int
    sport_name: str
    sport_slug: str
    stake: int
    status_name: str
    status_slug: str
    sub_account_id: Optional[str] = Field(default=None)  # TODO check type

    # noinspection PyMethodParameters
    @validator("actioned_at", "fixture_start_date", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    # noinspection PyMethodParameters
    @validator("customer_order_ref", pre=True)
    def parse_customer_order_ref(cls, v) -> Optional[CustomerOrderRef]:
        if v:
            return CustomerOrderRef(customer_order_ref=v)

    # noinspection PyMethodParameters
    @validator("customer_strategy_ref", pre=True)
    def parse_customer_strategy_ref(cls, v) -> Optional[CustomerStrategyRef]:
        if v:
            return CustomerStrategyRef(customer_strategy_ref=v)

    # noinspection PyMethodParameters
    @validator("price", pre=True)
    def price_parser(cls, v) -> Price:
        if isinstance(v, dict):
            return Price(
                price=v["decimal"],
                numerator=v["fraction"]["numerator"],
                denominator=v["fraction"]["denominator"],
            )
        else:
            raise TypeError(f"Expected value of dict")

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Competition: {self.competition_name}, Selection: {self.selection_name},Price: {self.price}, Stake: {self.stake}, Liability: {self.liability}"


class BetRequestStop(BaseResource):
    pending: bool

    def __repr__(self) -> str:
        return f"Pending: {self.pending}"


class MyBetsBetRequests(BaseResource):
    bets: List[MyActiveBet]
    bets_active: int
    last_page: int
    total_bets: int

    def __repr__(self) -> str:
        return f"Bets Active: {self.bets_active}"


class MyBetsBets(BaseResource):
    bets: List[MyActiveLayBet]
    bets_active: int
    last_page: int
    total_bets: int

    def __repr__(self) -> str:
        return f"Bets Active: {self.bets_active}"


class ActiveBet(BaseResource):
    bet_request_id: UUID
    bet_type_name: Optional[str] = Field(default=None)
    competition_name: str
    created_at: datetime
    customer_order_ref: Optional[CustomerOrderRef] = Field(default=None)
    customer_strategy_ref: Optional[CustomerStrategyRef] = Field(default=None)
    each_way_factor: Optional[float] = Field(default=None)
    fill_percentage: float
    fixture_id: Optional[int] = Field(default=None)
    fixture_name: str
    fixture_start_date: datetime = Field(alias="fixture_startdate")
    handicap: Optional[float] = Field(default=None)
    market_name: str
    matched_stake: float
    price: float
    price_denominator: int
    price_numerator: int
    selection_name: str
    sport_name: str
    stake: float
    status_name: str

    # noinspection PyMethodParameters
    @validator("customer_order_ref", pre=True)
    def parse_customer_order_ref(cls, v) -> Optional[CustomerOrderRef]:
        if v:
            return CustomerOrderRef(customer_order_ref=v)

    # noinspection PyMethodParameters
    @validator("customer_strategy_ref", pre=True)
    def parse_customer_strategy_ref(cls, v) -> Optional[CustomerStrategyRef]:
        if v:
            return CustomerStrategyRef(customer_strategy_ref=v)

    # noinspection PyMethodParameters
    @validator("created_at", "fixture_start_date", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __repr__(self) -> str:
        return f"Bet: {str(self.bet_request_id)}, Competition: {self.competition_name}, Selection: {self.selection_name}, Price: {self.price}, Stake: {self.stake}"


class ActiveBetRequests(BaseResource):
    bets: List[ActiveBet]
    bets_active: int
    last_page: int
    total_bets: int

    def __repr__(self) -> str:
        return f"Bets Active: {self.bets_active}"


class BetHistoryRequest(BaseResource):
    bets: List[BetHistory]
    last_page: int
    total_bets: int

    def __repr__(self) -> str:
        return f"Bets #: {len(self.bets)}"


class SelectionsForMarket(BaseResource):
    source_fixture_id: str
    source_market_id: str
    source_market_type_id: str
    source_selection_id: str
    trading_status: str
    name: str
    competitor_id: Optional[str] = Field(default=None)
    ut: datetime
    order: Optional[int] = Field(default=None)
    max_price: Optional[float] = Field(default=None)
    prices: List[Price]
    outcome: Optional[str] = Field(default=None)

    # noinspection PyMethodParameters
    @validator("ut", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")


class LineMarketsSelectionsForMarket(BaseResource):
    name: str
    display_name: str
    handicap: str
    line_data: List[SelectionsForMarket]


class Viewed(BaseResource):
    prev: UUID
    next: UUID

    def __repr__(self) -> str:
        return f"Prev: {self.prev}->Next: {self.next}"


class BetRequestMatchMore(BaseResource):
    matched: bool
    available: bool
    viewed: Viewed

    def __repr__(self) -> str:
        return f"Matched: {self.matched}, Available: {self.available}"


class LockBet(BaseResource):
    pass



from .baseclient import BaseClient
from requests.sessions import Session
from betconnect import endpoints
from betconnect.enums import Environment
from typing import Optional


class APIClient(BaseClient):
    def __init__(
        self,
        username: str,
        password: str,
        api_key: str,
        personalised_production_url: str,
        environment: Environment = Environment.PRODUCTION,
        session: Optional[Session] = None,
    ):
        """
        APIClient is used to make request to the betconnect API
        :param username: your betconnect username (string)
        :param password: your betconnect password (string)
        :param api_key: your betconnect api key (string)
        :param personalised_production_url: A production user supplied url. (account manager will supply this)
        :param environment: the environment endpoint you want to send requests to (enum Environment)
        :param session: Session object used in request default None. Session created if None with auth and headers handled.
        """
        self.betting = endpoints.Betting(self)
        self.account = endpoints.Account(self)
        super(APIClient, self).__init__(
            username=username,
            password=password,
            session=session,
            environment=environment,
            api_key=api_key,
            personalised_production_url=personalised_production_url,
        )



from betconnect import resources
from datetime import datetime


class TestAccountResources:
    def test_token(self):
        token = resources.Token(token="123456")
        assert token.token == "123456"

    def test_login(self):
        login = resources.Login(message="Login Successful", data={"token": "12345"})
        assert login.message == "Login Successful"
        assert isinstance(login.data, resources.Token)

    def test_account_preferences(self):

        account_preferences = resources.AccountPreferences(
            address_created="2021-06-16 05:52:24",
            address_line_1="1 London Road",
            address_line_3="London",
            address_updated="2021-06-16 05:52:24",
            admin_area="London",
            betconnect_pro=10,
            building="1",
            can_set_custom_odds=0,
            city="London",
            contact_number="11111111111",
            country="United Kingdom",
            country_iso2="GB",
            country_iso3="GBR",
            created="2021-06-16 05:52:24",
            default_home_page="dashboard",
            dob="2000-01-01",
            email="testemail@gmail.com",
            forename="Jim",
            full_name="Jim Bob",
            gamstop_result="N",
            kyc_result=1,
            last_login="2021-12-18 12:10:57",
            locality="London",
            marketing_terms_accepted=0,
            odds_format_decimal=0,
            page_size=25,
            pending_withdrawal=0,
            postcode="N1",
            seed_pro=0,
            surname="Bob",
            thoroughfare="London Road",
            user_category_id=1,
            user_id="a6a1gb91-6217-4e0d-b759-fbcaasb7a8ac",
            username="jbob",
            is_premium_subscriber=0,
        )
        assert isinstance(account_preferences.address_created, datetime)
        assert account_preferences.address_line_1 == "1 London Road"
        assert account_preferences.address_line_2 is None
        assert account_preferences.address_line_3 == "London"
        assert isinstance(account_preferences.address_updated, datetime)
        assert account_preferences.admin_area == "London"
        assert account_preferences.betconnect_pro == 10
        assert account_preferences.building == "1"
        assert account_preferences.can_set_custom_odds == 0
        assert account_preferences.city == "London"
        assert account_preferences.contact_number == "11111111111"
        assert account_preferences.country == "United Kingdom"
        assert account_preferences.country_iso2 == "GB"
        assert account_preferences.country_iso3 == "GBR"
        assert isinstance(account_preferences.created, datetime)
        assert account_preferences.default_home_page == "dashboard"
        assert account_preferences.display_name is None
        assert account_preferences.dob == "2000-01-01"
        assert account_preferences.email == "testemail@gmail.com"
        assert account_preferences.forename == "Jim"
        assert account_preferences.full_name == "Jim Bob"
        assert account_preferences.gamstop_result == "N"
        assert account_preferences.kyc_result == 1
        assert isinstance(account_preferences.last_login, datetime)
        assert account_preferences.locality == "London"
        assert account_preferences.kyc_result == 1
        assert account_preferences.odds_format_decimal == 0
        assert account_preferences.page_size == 25
        assert account_preferences.pending_withdrawal == 0
        assert account_preferences.pending_withdrawal_amount is None
        assert account_preferences.postcode == "N1"
        assert account_preferences.premise is None
        assert account_preferences.seed_pro == 0
        assert account_preferences.surname == "Bob"
        assert account_preferences.thoroughfare == "London Road"
        assert account_preferences.user_category_id == 1
        assert account_preferences.user_id == "a6a1gb91-6217-4e0d-b759-fbcaasb7a8ac"
        assert account_preferences.username == "jbob"
        assert account_preferences.is_premium_subscriber == 0

        account_preferences = resources.AccountPreferences(
            address_created="2021-06-16 05:52:24",
            address_line_1="1 London Road",
            address_updated="2021-06-16 05:52:24",
            admin_area=None,
            betconnect_pro=10,
            can_set_custom_odds=0,
            contact_number="11111111111",
            country="United Kingdom",
            country_iso2="GB",
            country_iso3="GBR",
            created="2021-06-16 05:52:24",
            default_home_page="dashboard",
            dob="2000-01-01",
            email="testemail@gmail.com",
            forename="Jim",
            full_name="Jim Bob",
            gamstop_result="N",
            kyc_result=1,
            last_login="2021-12-18 12:10:57",
            locality=None,
            marketing_terms_accepted=0,
            odds_format_decimal=0,
            page_size=25,
            pending_withdrawal=0,
            postcode="N1",
            seed_pro=0,
            surname="Bob",
            user_category_id=1,
            user_id="a6a1gb91-6217-4e0d-b759-fbcaasb7a8ac",
            username="jbob",
            is_premium_subscriber=0,
        )
        assert account_preferences.address_line_2 is None
        assert account_preferences.address_line_3 is None
        assert account_preferences.admin_area is None
        assert account_preferences.locality is None
        assert account_preferences.city is None
        assert account_preferences.thoroughfare is None
        assert account_preferences.building is None



from enum import Enum


class Environment(Enum):
    PRODUCTION = "Production"
    STAGING = "Staging"


class BetSide(Enum):
    BACK = "back"
    LAY = "lay"


class BetRequestStatus(Enum):
    ACTIVE = "active"
    SETTLED = "settled"


class MarketStatus(Enum):
    OPEN = "Open"
    SUSPENDED = "Suspended"
    CLOSED = "Closed"


class TradingStatus(Enum):
    OPEN = "Open"
    ACTIVE = "Active"
    CLOSED = "Closed"
    NON_RUNNER = "NonRunner"


class BetStatus(Enum):
    RECEIVED = "received"
    CANCELLED = "cancelled"
    REJECTED = "rejected"
    VOIDED_BY_PRO = "voided_by_pro"
    VOIDED_BY_ADMIN = "voided_by_admit"
    MATCHED = "matched"
    MATCHED_MORE = "matched_more"
    EXPIRED = "expired"



from betconnect import utils
import pytest
from uuid import UUID
from betconnect import resources


class TestUtils:
    def test_is_valid_uuid(self):
        assert utils.is_valid_uuid(uuid_to_test="123243") is False
        assert utils.is_valid_uuid("c9bf9e57-1685-4c89-bafb-ff5af830be8a")

    def test_calculate_book_percentage(self, mock_selections_for_market_response):
        all_selections = [
            resources.SelectionsForMarket.create_from_dict(s)
            for s in mock_selections_for_market_response[1]["data"]
        ]
        assert (
            utils.calculate_book_percentage(market_selections=all_selections) == 1.222
        )

    def test_parse_bet_request_id(self):
        assert isinstance(
            utils.parse_bet_request_id("c9bf9e57-1685-4c89-bafb-ff5af830be8a"), UUID
        )
        with pytest.raises(Exception):
            utils.parse_bet_request_id("c9bf9e57-1685-4c89-bafb-ff5a")



from __future__ import annotations
from typing import TYPE_CHECKING, Union, List, Type, Tuple
from datetime import datetime, timedelta
from betconnect.resources.baseresource import BaseResource
import requests
import json
import logging
import time
from requests import Response
from betconnect import resources
from betconnect.exceptions import APIError
from uuid import UUID
from betconnect import config

logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from betconnect.apiclient import APIClient


class BaseEndpoint:
    _read_timeout = config.CLIENT_READOUT_TIME_SECS

    def __init__(self, client: APIClient):
        self.client = client

    @property
    def session(self) -> requests.Session:
        return self.client.session

    @property
    def today(self):
        return datetime.utcnow()

    @property
    def api_version(self) -> str:
        return self.client.api_version

    @property
    def tomorrow(self):
        return datetime.utcnow() + timedelta(days=1)

    def _request(
        self, method_uri: str, params: dict = None, authenticated: bool = False
    ) -> Tuple[requests.Response, dict, float]:
        """
        :param str method_uri: uri to be used, defined by each function.
        :param dict params: Query Params to be used in request
        :param bool authenticated: If the request requires the user to be logged in
        :return: tuple of the Response, dict (json_dict), float (elapsed time)
        """
        params = params if params else {}

        uri = self.client.uri + method_uri
        time_sent = time.time()

        logger.debug(f"Requesting the data for {uri}")

        if authenticated and self.client.logged_in is False:
            logger.info(
                f"Need to be logged in before accessing{uri}. Attempting login with supplied credentials"
            )
            self.client.account.login()

        try:
            response = self.session.get(uri, timeout=self._read_timeout, params=params)
        except requests.ConnectionError as e:
            raise APIError(None, uri, params, e)
        except Exception as e:
            raise APIError(None, uri, params, e)
        elapsed_time = time.time() - time_sent

        response_json = self.load_json_content(response)

        return response, response_json, elapsed_time

    def _post(
        self, method_uri: str, data: dict, authenticated: bool = True
    ) -> Tuple[requests.Response, dict, float]:
        """
        :param str method_uri: uri to be used, defined by each function.
        :param dict data: body of data
        :param bool authenticated: If the request requires the user to be logged in
        :return: tuple of the Response, dict (json_dict), float (elapsed time)
        """
        uri = self.client.uri + method_uri
        time_sent = time.time()
        logger.debug(f"posting the data for {uri}")

        if authenticated and self.client.logged_in is False:
            logger.info(
                f"Need to be logged in before accessing{uri}. Attempting login with supplied credentials"
            )
            self.client.account.login()

        try:
            response = self.session.post(uri, timeout=self._read_timeout, json=data)
        except requests.ConnectionError as e:
            raise APIError(None, uri, data, e)
        except Exception as e:
            raise APIError(None, uri, data, e)
        elapsed_time = time.time() - time_sent

        response_json = self.load_json_content(response)

        return response, response_json, elapsed_time

    def _patch(
        self, method_uri: str, data: dict
    ) -> Tuple[requests.Response, dict, float]:
        """
        :param str method_uri: uri to be used, defined by each function.
        :param dict data: body of data
        :return: tuple of the Response, dict (json_dict), float (elapsed time)
        """
        uri = self.client.uri + method_uri
        time_sent = time.time()
        logger.debug(f"patching the data for {uri}")

        if self.client.logged_in is False:
            logger.info(
                f"Need to be logged in before accessing{uri}. Attempting login with supplied credentials"
            )
            self.client.account.login()

        try:
            response = self.session.patch(uri, timeout=self._read_timeout, json=data)
        except requests.ConnectionError as e:
            raise APIError(None, uri, data, e)
        except Exception as e:
            raise APIError(None, uri, data, e)
        elapsed_time = time.time() - time_sent

        response_json = self.load_json_content(response)

        return response, response_json, elapsed_time

    @staticmethod
    def process_request_exception(
        response: requests.Response, response_json: dict
    ) -> resources.BaseRequestException:
        logger.exception(
            f"Issue with request for: {response.url}, message: {response_json.get('message')}"
        )
        return resources.BaseRequestException(
            message=response_json.get("message"),
            request_url=response.url,
            status_code=response.status_code,
        )

    @staticmethod
    def check_bet_request_id(bet_request_id: UUID):
        if isinstance(bet_request_id, UUID) is False:
            raise Exception(f"Incorrect UUID supplied for the bet_request_id")

    @staticmethod
    def load_json_content(response: requests.Response):
        if response.headers["content-type"] in [
            "application/json",
            "application/json; charset=utf-8",
        ]:
            return json.loads(response.content.decode("utf-8"))
        else:
            logger.exception("Unrecognised content encoding type")

    def process_response(
        self,
        response: requests.Response,
        response_json: Union[dict, list],
        resource: Type[BaseResource],
        elapsed_time: float,
    ) -> Union[BaseResource, dict, list, resources.BaseRequestException]:
        """
        Process the endpoint function responses from betconnect, parsing the data to the relevant resources
        :param response: The request response
        :param response_json: The json data from the response
        :param resource: The resource to parse to the data too
        :param elapsed_time: The time taken to make the request
        :return: A resource for the response data or a BaseRequestException if BetConnect has detected an issue with
        the request.
        """

        if self.check_status_code(response):
            if "data" in response_json:
                data = response_json["data"]

                logger.debug(
                    f"It took {elapsed_time} s to request the data for url {response.url}"
                )

                if isinstance(data, dict):
                    return resource.create_from_dict(data)
                elif isinstance(data, list):
                    if len(data) == 0:
                        logger.info(f"No data could be found for {response.url}")
                    return [resource.create_from_dict(r) for r in data]
                else:
                    raise Exception(
                        "Unkown response data type"
                    )  # TODO add specific Exception Here
            elif "message" in response_json:
                return resources.ResponseMessage(
                    message=response_json.get("message"),
                    status_code=response.status_code,
                    request_url=response.url,
                )
            else:
                raise Exception("Expected response json to contain data key")

        else:
            return self.process_request_exception(response, response_json)

    def _put(
        self, method_uri: str, data: dict, authenticated: bool = True
    ) -> Tuple[requests.Response, dict, float]:
        """
        :param str method_uri: uri to be used, defined by each function.
        :param dict data: body of data
        :param bool authenticated: If the request requires the user to be logged in
        :return: tuple of the Response, dict (json_dict), float (elapsed time)
        """
        uri = self.client.uri + method_uri
        time_sent = time.time()
        logger.debug(f"putting the data for {uri}")

        if authenticated and self.client.logged_in is False:
            logger.info(
                f"Need to be logged in before accessing{uri}. Attempting login with supplied credentials"
            )
            self.client.account.login()

        try:
            response = self.session.put(uri, timeout=self._read_timeout, json=data)
        except requests.ConnectionError as e:
            raise APIError(None, uri, data, e)
        except Exception as e:
            raise APIError(None, uri, data, e)
        elapsed_time = time.time() - time_sent

        response_json = self.load_json_content(response)

        return response, response_json, elapsed_time

    @staticmethod
    def check_status_code(response: Response, codes: List[int] = None) -> bool:
        """
        Checks the reponse for valid status codes. Valid status codes {200} + supplied codes
        :param response: A response resource (from the request)
        :param codes: A list of accepted codes
        :return: A bool validating the code
        """
        codes = codes if codes else []
        return (
            True
            if (response.status_code in codes) or (response.status_code == 200)
            else False
        )



import logging
from decouple import config
from betconnect import enums
from betconnect import resources
from betconnect.apiclient import APIClient
from betconnect.enums import Environment
from datetime import datetime

logger = logging.getLogger(__name__)

"""
This script is an example workflow for stopping an active betrequest.
PLEASE MAKE SURE YOU ARE ONLY PRACTICING ORDERS AGAINST STAGING ENVIRONMENTS IF YOU HAVE NOT FULLY TESTED YOUR APP!
"""

# Create a trading client instance
client = APIClient(
    username=config("STAGING_BETCONNECT_USERNAME"),
    password=config("STAGING_BETCONNECT_PASSWORD"),
    api_key=config("STAGING_BETCONNECT_API_KEY"),
    environment=Environment.STAGING,
    personalised_production_url=config("PRODUCTION_URI"),
)

# login
client.account.login()

assert client.environment == Environment.STAGING

# Get active bets
my_active_back_bets = client.betting.my_bets(
    side=enums.BetSide.BACK, status=enums.BetRequestStatus.ACTIVE
)

cancellation_responses = []

if len(my_active_back_bets.bets) > 0:
    for bet in my_active_back_bets.bets[::-1]:  # type: resources.MyActiveBet
        if bet.fixture_start_date:
            if bet.fixture_start_date > datetime.utcnow():
                bet_stop_response = client.betting.bet_request_stop(
                    bet_request_id=bet.bet_request_id,
                    stop_bet_reason="The bet was too good!",
                )
                cancellation_responses.append(bet_stop_response)



from .baseresource import BaseResource
from datetime import datetime
from pydantic import validator, Field
from typing import Optional


class Token(BaseResource):
    token: str

    def __repr__(self):
        return f"Token: {self.token}"


class Login(BaseResource):
    message: str
    data: Token

    def __repr__(self) -> str:
        return f"Login Message: {self.message}"


class AccountPreferences(BaseResource):
    address_created: datetime
    address_line_1: str
    address_line_2: Optional[str] = Field(default=None)
    address_line_3: Optional[str] = Field(default=None)
    address_updated: datetime
    admin_area: Optional[str] = Field(default=None)
    betconnect_pro: int
    building: Optional[str] = Field(default=None)
    can_set_custom_odds: int
    city: Optional[str] = Field(default=None)
    contact_number: str
    country: str
    country_iso2: str
    country_iso3: str
    created: datetime
    default_home_page: str
    display_name: Optional[str] = Field(default=None)
    dob: str
    email: str
    forename: str
    full_name: str = Field(alias="fullname")
    gamstop_result: str
    kyc_result: int
    last_login: datetime
    locality: Optional[str] = Field(default=None)
    marketing_terms_accepted: int
    odds_format_decimal: int
    page_size: int
    pending_withdrawal: int
    pending_withdrawal_amount: Optional[float] = Field(default=None)
    postcode: str
    premise: Optional[str] = Field(default=None)
    seed_pro: int
    surname: str
    thoroughfare: Optional[str] = Field(default=None)
    user_category_id: int
    user_id: str
    username: str
    is_premium_subscriber: int

    # noinspection PyMethodParameters
    @validator("address_created", "address_updated", "created", pre=True)
    def date_parser(cls, v) -> datetime:
        if isinstance(v, str):
            return datetime.fromisoformat(v)
        elif isinstance(v, datetime):
            return v
        else:
            raise TypeError(f"Expected value of type str or datetime")

    def __repr__(self) -> str:
        return f"Username: {self.username} ({self.user_id})"



from betconnect import resources
import pytest
from betconnect import exceptions


class TestFilters:
    def test_filter(self):
        request_filter = resources.Filter()
        assert isinstance(request_filter, resources.Filter)

    # noinspection SpellCheckingInspection
    def test_get_bet_request_filter(self):
        request_filter = resources.GetBetRequestFilter(sport_id=1)
        assert request_filter.sport_id == 1
        assert request_filter.bookmakers == []
        assert request_filter.horse_racing_regions == []
        assert request_filter.min_odds == 1.01
        assert request_filter.max_odds == 1000
        assert request_filter.accept_each_way == True
        assert request_filter.bet_request_id is None

        # test length == 36
        with pytest.raises(exceptions.BetRequestIDFormatException):
            resources.GetBetRequestFilter(bet_request_id="invalid bet request length")

        # test uuid format
        with pytest.raises(exceptions.BetRequestIDFormatException):
            resources.GetBetRequestFilter(
                bet_request_id="c9bf9e575168554c895bafb5ff5af830be8a"
            )
        # test min odds validation
        with pytest.raises(exceptions.MinOddException):
            resources.GetBetRequestFilter(min_odds=1)

    def test_create_bet_request_by_selection_filter(self):
        request_filter = resources.CreateBetRequestBySelectionFilter(
            selection_id=123,
            price=10,
            stake=50,
            bet_type="WIN",
        )
        assert request_filter.selection_id == 123
        assert request_filter.price == 10
        assert request_filter.stake == 50
        assert request_filter.bet_type == "WIN"
        assert request_filter.handicap is None
        assert request_filter.customer_strategy_ref is None
        assert request_filter.customer_order_ref is None

        # test min stake validation
        with pytest.raises(exceptions.BetRequestIDStakeSizeException):
            resources.CreateBetRequestBySelectionFilter(
                selection_id=123,
                price=10,
                stake=0.99,
                bet_type="WIN",
            )

        # test customer_strategy_ref length
        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            resources.CreateBetRequestBySelectionFilter(
                selection_id=123,
                price=10,
                stake=10,
                bet_type="WIN",
                customer_strategy_ref="11111111111111111111111111111111111",
            )

        # test validate_customer_order_ref length
        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            resources.CreateBetRequestBySelectionFilter(
                selection_id=123,
                price=10,
                stake=10,
                bet_type="WIN",
                customer_order_ref="1111111111111111111111111111111111111111111111111111",
            )

    def test_create_bet_request_by_competitor_filter(self):
        request_filter = resources.CreateBetRequestByCompetitorFilter(
            fixture_id=1,
            market_type_id=1,
            competitor="123",
            price=10,
            stake=50,
            bet_type="WIN",
        )
        assert request_filter.fixture_id == 1
        assert request_filter.market_type_id == 1
        assert request_filter.competitor == "123"
        assert request_filter.price == 10
        assert request_filter.stake == 50
        assert request_filter.bet_type == "WIN"
        assert request_filter.handicap is None
        assert request_filter.customer_strategy_ref is None
        assert request_filter.customer_order_ref is None

        # test min stake validation
        with pytest.raises(exceptions.BetRequestIDStakeSizeException):
            resources.CreateBetRequestByCompetitorFilter(
                fixture_id=1,
                market_type_id=1,
                competitor="123",
                price=10,
                stake=0.99,
                bet_type="WIN",
            )

        # test customer_strategy_ref length
        with pytest.raises(
            exceptions.BetRequestInvalidCustomerStrategyRefFormatException
        ):
            resources.CreateBetRequestByCompetitorFilter(
                fixture_id=1,
                market_type_id=1,
                competitor="123",
                price=10,
                stake=10,
                bet_type="WIN",
                customer_strategy_ref="11111111111111111111111111111111111",
            )

        # test validate_customer_order_ref length
        with pytest.raises(exceptions.BetRequestInvalidCustomerOrderRefFormatException):
            resources.CreateBetRequestByCompetitorFilter(
                fixture_id=1,
                market_type_id=1,
                competitor="123",
                price=10,
                stake=10,
                bet_type="WIN",
                customer_order_ref="1111111111111111111111111111111111111111111111111111",
            )

    def test_create_bet_request_filter_alias_working(self):
        request_filter = resources.CreateBetRequestFilter(
            fixture_id=1,
            market_type_id=1,
            competitor="123",
            price=10,
            stake=50,
            bet_type="WIN",
        )
        assert request_filter.fixture_id == 1
        assert request_filter.market_type_id == 1
        assert request_filter.competitor == "123"
        assert request_filter.price == 10
        assert request_filter.stake == 50
        assert request_filter.bet_type == "WIN"
        assert request_filter.handicap is None
        assert request_filter.customer_strategy_ref is None
        assert request_filter.customer_order_ref is None
        assert isinstance(request_filter, resources.CreateBetRequestByCompetitorFilter)
